@@ -1,8 +1,8 @@
-(def tk-version "1.1.0")
+(def tk-version "1.1.1")
 (def ks-version "1.0.0")
 (def jetty-version "9.2.10.v20150310")
 
-(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.3.0"
+(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.5.0"
   :description "A jetty9-based webserver implementation for use with the puppetlabs/trapperkeeper service framework."
   :url "https://github.com/puppetlabs/trapperkeeper-webserver-jetty9"
   :license {:name "Apache License, Version 2.0"
@@ -13,11 +13,13 @@
   :pedantic? :abort
   :dependencies [[org.clojure/clojure "1.6.0"]
                  [org.clojure/tools.logging "0.2.6"]
-                 [clj-time "0.5.1"]
-                 [prismatic/schema "0.2.2"]
-                 [puppetlabs/kitchensink ~ks-version]
-                 [puppetlabs/trapperkeeper ~tk-version]
-                 [puppetlabs/ssl-utils "0.8.0"]
+                 [prismatic/schema "0.4.0"]
+                 [prismatic/plumbing "0.4.2"]
+
+                 ;; Let ssl-utils bring in the clj-time dependency
+                 [puppetlabs/ssl-utils "0.8.1"]
+                 [puppetlabs/kitchensink ~ks-version :exclusions [clj-time]]
+                 [puppetlabs/trapperkeeper ~tk-version :exclusions [clj-time]]
 
                  [ch.qos.logback/logback-access "1.1.1"]
 
@@ -33,10 +35,11 @@
                  [org.eclipse.jetty/jetty-webapp ~jetty-version]
                  [org.eclipse.jetty/jetty-proxy ~jetty-version]
                  [org.eclipse.jetty/jetty-jmx ~jetty-version]
+                 [org.eclipse.jetty.websocket/websocket-server ~jetty-version]
 
                  [ring/ring-servlet "1.1.8" :exclusions [javax.servlet/servlet-api commons-codec]]]
 
-  :plugins [[lein-release "1.0.5"]]
+  :plugins [[lein-release "1.0.5" :exclusions [org.clojure/clojure]]]
 
   :lein-release {:scm         :git
                  :deploy-via  :lein-deploy}
@@ -61,15 +64,18 @@
                                   "examples/webrouting_app/src"]
                    :java-source-paths ["examples/servlet_app/src/java"
                                        "test/java"]
-                   :dependencies [[puppetlabs/http-client "0.4.3"]
+                   :dependencies [[puppetlabs/http-client "0.4.4"]
                                   [puppetlabs/kitchensink ~ks-version :classifier "test"]
                                   [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
-                                  [org.clojure/tools.namespace "0.2.4"]
+                                  [org.clojure/tools.namespace "0.2.10"]
                                   [org.clojure/java.jmx "0.2.0"]
                                   [spyscope "0.1.4"]
                                   [compojure "1.1.8" :exclusions [ring/ring-core
                                                                   commons-io
-                                                                  org.clojure/tools.macro]]]
+                                                                  org.clojure/tools.macro]]
+                                  [stylefruits/gniazdo "0.4.0" :exclusions [org.eclipse.jetty.websocket/websocket-api
+                                                                            org.eclipse.jetty.websocket/websocket-client
+                                                                            org.eclipse.jetty/jetty-util]]]
                     :injections [(require 'spyscope.core)]
                     ;; Enable SSLv3 for unit tests that exercise SSLv3
                     :jvm-opts ["-Djava.security.properties=./dev-resources/java.security"]}
