@@ -1,4 +1,4 @@
-(def pdb-version "3.2.4")
+(def pdb-version "4.0.0")
 
 (defn deploy-info
   "Generate deployment information from the URL supplied and the username and
@@ -9,9 +9,10 @@
    :password :env/nexus_jenkins_password
    :sign-releases false})
 
-(def tk-version "1.1.1")
-(def tk-jetty9-version "1.3.1")
-(def ks-version "1.1.0")
+(def tk-version "1.2.0")
+(def tk-jetty9-version "1.5.0")
+(def ks-version "1.2.0")
+(def tk-status-version "0.3.1")
 
 (def pdb-jvm-opts
   (case (System/getProperty "java.specification.version")
@@ -31,17 +32,17 @@
   ;; requires lein 2.2.0+.
   :pedantic? :abort
 
-  :dependencies [[org.clojure/clojure "1.7.0"]
-                 [cheshire "5.4.0"]
-                 [org.clojure/core.match "0.2.2"]
-                 [org.clojure/math.combinatorics "0.0.4"]
+  :dependencies [[org.clojure/clojure "1.8.0"]
+                 [cheshire "5.5.0"]
+                 [org.clojure/core.match "0.3.0-alpha4" :exclusions [org.clojure/tools.analyzer.jvm]]
+                 [org.clojure/math.combinatorics "0.1.1"]
                  [org.clojure/math.numeric-tower "0.0.4"]
                  [org.clojure/tools.logging "0.3.1"]
                  [org.clojure/core.memoize "0.5.8"]
                  [puppetlabs/tools.namespace "0.2.4.1"]
                  [clj-stacktrace "0.2.8"]
-                 [metrics-clojure "0.7.0" :exclusions [org.clojure/clojure org.slf4j/slf4j-api]]
-                 [clj-time "0.9.0"]
+                 [metrics-clojure "2.6.1" :exclusions [org.clojure/clojure org.slf4j/slf4j-api]]
+                 [clj-time "0.11.0"]
                  [org.clojure/java.jmx "0.3.1"]
                  ;; Filesystem utilities
                  [me.raynes/fs "1.4.5"]
@@ -54,55 +55,53 @@
                  [slingshot "0.12.2"]
 
                  ;; Database connectivity
-                 [com.jolbox/bonecp "0.7.1.RELEASE" :exclusions [org.slf4j/slf4j-api]]
-                 [org.clojure/java.jdbc "0.3.7"]
-                 [org.hsqldb/hsqldb "2.2.8"]
-                 [org.postgresql/postgresql "9.2-1003-jdbc4"]
+                 [com.zaxxer/HikariCP "2.4.3"]
+                 [org.clojure/java.jdbc "0.4.2"]
+                 [org.postgresql/postgresql "9.4-1206-jdbc4"]
 
                  ;; MQ connectivity
-                 [org.apache.activemq/activemq-broker "5.13.0"]
-                 [org.apache.activemq/activemq-kahadb-store "5.13.0"]
-                 [org.apache.activemq/activemq-pool "5.13.0"]
+                 [org.apache.activemq/activemq-broker "5.13.2" :exclusions [org.slf4j/slf4j-api]]
+                 [org.apache.activemq/activemq-kahadb-store "5.13.2" :exclusions [org.slf4j/slf4j-api]]
+                 [org.apache.activemq/activemq-pool "5.13.2" :exclusions [org.slf4j/slf4j-api]]
+
+                 ;; Parsing library required by PQL
+                 [instaparse "1.4.1"]
 
                  ;; bridge to allow some spring/activemq stuff to log over slf4j
-                 [org.slf4j/jcl-over-slf4j "1.7.10"]
+                 [org.slf4j/jcl-over-slf4j "1.7.14" :exclusions [org.slf4j/slf4j-api]]
                  ;; WebAPI support libraries.
-                 [net.cgrand/moustache "1.1.0" :exclusions [ring/ring-core org.clojure/clojure]]
-                 [compojure "1.3.3"]
-                 [clj-http "1.0.1"]
-                 [ring/ring-core "1.3.2" :exclusions [javax.servlet/servlet-api]]
-                 [org.apache.commons/commons-compress "1.8"]
+                 [compojure "1.4.0"]
+                 [clj-http "2.0.1"]
+                 [ring/ring-core "1.4.0" :exclusions [javax.servlet/servlet-api org.clojure/tools.reader]]
+                 [org.apache.commons/commons-compress "1.10"]
                  [puppetlabs/kitchensink ~ks-version]
                  [puppetlabs/trapperkeeper ~tk-version]
                  [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty9-version]
-                 [prismatic/schema "0.4.1"]
+                 [prismatic/schema "1.0.4"]
+                 [trptcolin/versioneer "0.2.0"]
+                 [puppetlabs/trapperkeeper-status ~tk-status-version]
                  [org.clojure/tools.macro "0.1.5"]
                  [com.novemberain/pantomime "2.1.0"]
                  [fast-zip-visit "1.0.2"]
                  [robert/hooke "1.3.0"]
-                 [honeysql "0.5.2"]
-                 [org.clojure/data.xml "0.0.8"]
+                 [honeysql "0.6.3"]
                  [com.rpl/specter "0.5.7"]
-                 [org.clojure/core.async "0.1.346.0-17112a-alpha"]
-                 [puppetlabs/http-client "0.4.4"]]
+                 [org.clojure/core.async "0.2.374"]
+                 [puppetlabs/http-client "0.5.0" :exclusions [org.apache.httpcomponents/httpclient
+                                                              org.apache.httpcomponents/httpcore
+                                                              org.slf4j/slf4j-api]]
+                 [com.taoensso/nippy "2.10.0" :exclusions [org.clojure/tools.reader]]
+                 [bidi "1.25.1" :exclusions [org.clojure/clojurescript]]
+                 [puppetlabs/comidi "0.3.1"]]
 
   :jvm-opts ~pdb-jvm-opts
 
-  ;;The below test-selectors is basically using the PUPPETDB_DBTYPE
-  ;;environment variable to be the test selector.  The selector below
-  ;;will always run a test, unless it has a meta value for that
-  ;;dbtype, and that value is falsey, such as
-  ;;(deftest ^{:postgres false} my-test-name...)
-
-  :test-selectors {:default (fn [test-var-meta]
-                              (let [dbtype (or (keyword (System/getenv "PUPPETDB_DBTYPE"))
-                                               :postgres)]
-                                (get test-var-meta dbtype true)))}
-
   :repositories [["releases" "http://nexus.delivery.puppetlabs.net/content/repositories/releases/"]
                  ["snapshots" "http://nexus.delivery.puppetlabs.net/content/repositories/snapshots/"]]
 
-  :plugins [[lein-release "1.0.5"]]
+  :plugins [[lein-release "1.0.5"]
+            [lein-cloverage "1.0.6" :exclusions [org.clojure/clojure]]]
+
   :lein-release {:scm        :git
                  :deploy-via :lein-deploy}
 
@@ -129,9 +128,11 @@
                                   [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
                                   [puppetlabs/kitchensink ~ks-version :classifier "test"]
                                   [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty9-version :classifier "test"]
-                                  [org.flatland/ordered "1.5.2"]
+                                  [org.flatland/ordered "1.5.3"]
                                   [org.clojure/test.check "0.5.9"]
-                                  [environ "1.0.0"]]}
+                                  [environ "1.0.2"]
+                                  [org.clojure/tools.cli "0.3.3"] ; prevents dependency clash caused by lein-cloverage
+                                  [riddley "0.1.12"]]}
              :ezbake {:dependencies ^:replace [[puppetlabs/puppetdb ~pdb-version]
                                                [org.clojure/tools.nrepl "0.2.3"]]
                       :name "puppetdb"
