@@ -67,7 +67,6 @@
             [puppetlabs.puppetdb.schema :refer [defn-validated]]
             [puppetlabs.puppetdb.utils :as utils]
             [slingshot.slingshot :refer [try+ throw+]]
-            [cheshire.custom :refer [JSONable]]
             [puppetlabs.puppetdb.command.constants :refer [command-names]]
             [puppetlabs.trapperkeeper.services
              :refer [defservice service-context]]
@@ -78,73 +77,6 @@
             [clojure.set :as set]
             [clojure.core.async :as async]))
 
-;; ## Command parsing
-
-(defmulti parse-command
-  "Take a wire-format command and parse it into a command object."
-  (comp class :body))
-
-(defmethod parse-command (class (byte-array 0))
-  [bytes-message]
-  (parse-command (update-in bytes-message [:body] #(String. % "UTF-8"))))
-
-(defmethod parse-command String
-  [{:keys [headers body]}]
-  {:pre  [(string? body)]
-   :post [(map? %)
-          (:payload %)
-          (string? (:command %))
-          (number? (:version %))
-          (map? (:annotations %))]}
-  (let [message     (json/parse-string body true)
-        received    (get headers :received (kitchensink/timestamp))
-        id          (get headers :id (kitchensink/uuid))]
-    (-> message
-        (assoc-in [:annotations :received] received)
-        (assoc-in [:annotations :id] id))))
-
-(defn assemble-command
-  "Builds a command-map from the supplied parameters"
-  [command version payload]
-  {:pre  [(string? command)
-          (number? version)
-          (satisfies? JSONable payload)]
-   :post [(map? %)
-          (:payload %)]}
-  {:command command
-   :version version
-   :payload payload})
-
-
-;; ## Command submission
-
-(defn-validated ^:private do-enqueue-raw-command :- s/Str
-  "Submits raw-command to the mq-endpoint of mq-connection and returns
-  its id."
-  [mq-connection :- mq/connection-schema
-   mq-endpoint :- s/Str
-   raw-command :- s/Str
-   uuid :- (s/maybe s/Str)]
-  (let [uuid (or uuid (kitchensink/uuid))]
-    (mq/send-message! mq-connection mq-endpoint
-                      raw-command
-                      {"received" (kitchensink/timestamp) "id" uuid})
-    uuid))
-
-(defn-validated ^:private do-enqueue-command :- s/Str
-  "Submits command to the mq-endpoint of mq-connection and returns
-  its id. Annotates the command via annotate-command."
-  [mq-connection :- mq/connection-schema
-   mq-endpoint :- s/Str
-   command :- s/Str
-   version :- s/Int
-   payload
-   uuid :- (s/maybe s/Str)]
-  (let [command-map (assemble-command command version payload)]
-    (do-enqueue-raw-command mq-connection mq-endpoint (json/generate-string command-map) uuid)))
-
-;; ## Command processing exception classes
-
 (defn fatality
   "Create an object representing a fatal command-processing exception
 
@@ -160,118 +92,230 @@
     (catch Throwable e#
       (throw+ (fatality e#)))))
 
-;; ## Command processors
+(defn version-range [min-version max-version]
+  (set (range min-version (inc max-version))))
 
-(defmulti process-command!
-  "Takes a command object and processes it to completion. Dispatch is
-  based on the command's name and version information"
-  (fn [{:keys [command version]} _]
-    [command version]))
+(def supported-command-versions
+  {"replace facts" (version-range 2 4)
+   "replace catalog" (version-range 4 8)
+   "store report" (version-range 3 7)
+   "deactivate node" (version-range 1 3)})
+
+(defn- die-on-header-payload-mismatch
+  [name in-header in-body]
+  (when (and in-header in-body (not= in-header in-body))
+    (throw+
+     (fatality
+      (Exception.
+       (format "%s mismatch between message properties and payload (%s != %s)"
+               name in-header in-body))))))
+
+(def new-message-schema
+  {:headers {:command s/Str
+             :version s/Str
+             :certname s/Str
+             :received s/Str
+             :id s/Str}
+   :body (s/cond-pre s/Str utils/byte-array-class)})
+
+(def queue-message-schema
+  ;; Perhaps eventually require :id and :received
+  {(s/optional-key :headers) {(s/optional-key :id) s/Str
+                              (s/optional-key :received) s/Str
+                              (s/optional-key :scheduledJobId) s/Str
+                              (s/optional-key :JMSXDeliveryCount) s/Str
+                              s/Any s/Any}
+   :body (s/cond-pre s/Str utils/byte-array-class)})
+
+(def new-command-schema ;; This could probably be stricter
+  {:command s/Str
+   :version s/Int
+   :certname s/Str
+   :payload {s/Any s/Any}
+   :annotations
+   (s/pred
+    (fn [x]
+      (and (map? x)
+           (some-> x :id string?)
+           (some-> x :received string?)
+           (empty? (select-keys x [:command :version :certname])))))})
+
+(def queue-command-schema
+  ;; Created to match existing test behavior, but is that
+  ;; comprehensive? i.e. if we're wrong, and this is too strict, then
+  ;; it could break on outside queue content on release.  Adding a
+  ;; schema here could change our effectively public queue API.
+  (-> new-command-schema
+      (dissoc :certname)
+      (assoc (s/optional-key :certname) s/Str)
+      (dissoc :payload)
+      (assoc (s/optional-key :payload) (s/cond-pre s/Str {s/Any s/Any}))))
+
+(defn-validated ^:private parse-new-command :- new-command-schema
+  "Parses a new-format queue command and returns it in the traditional
+  format (see parse-queue-command).  New-style messages must have all
+  5 headers (command, version, certname, received, and id), and the
+  body must be the bare command content (i.e. the payload).  So for a
+  \"deactivate node\" command, a string like
+  {\"certname\":\"test1\",\"producer_timestamp\":\"2015-01-01\"}."
+  [{:keys [headers body]} :- new-message-schema]
+  (let [{:keys [command version certname received id]} headers
+        version (Integer/parseInt version)]
+    (let [message (json/parse body true)]
+      (die-on-header-payload-mismatch "certname"
+                                      certname
+                                      (get-in message ["payload" "certname"]))
+      ;; Since new commands aren't the queue retry format, we expect
+      ;; no annotations.
+      (assert (not (:annotations message)))
+      {:command command
+       :version version
+       :certname certname ;; Duplicated with respect to payload for now.
+       :payload message
+       :annotations (dissoc headers :command :version :certname)})))
+
+(defn-validated ^:private parse-queue-command :- queue-command-schema
+  "Parses a traditional queue command (also the current format for
+  failed, re-queued messages).  See parse-new-command for the handling
+  of newly received messages.  These messages must not have command,
+  version, or certname headers, and the body must be a JSON map like
+  this: {\"command\":\"deactivate node\",\"version\":3,\"payload\":{\"certname\":...}}."
+  [{:keys [headers body]} :- queue-message-schema]
+  (let [{:keys [command version certname]} headers]
+    (update (json/parse-strict body true) :annotations merge
+            {:received (kitchensink/timestamp) :id (kitchensink/uuid)}
+            headers)))
+
+(defn parse-command
+  "Parses a queue message and returns it as a command map."
+  [{:keys [headers body] :as message}]
+  {:post [(map? %)
+          (:payload %)
+          (string? (:command %))
+          (number? (:version %))
+          (map? (:annotations %))]}
+  (if (:command headers)
+    (parse-new-command message)
+    (parse-queue-command message)))
+
+;; ## Command submission
+
+(defn-validated ^:private do-enqueue-raw-command :- s/Str
+  "Submits raw-command to the mq-endpoint of mq-connection and returns
+  its id."
+  [mq-connection :- mq/connection-schema
+   mq-endpoint :- s/Str
+   raw-command :- (s/cond-pre s/Str utils/byte-array-class)
+   uuid :- (s/maybe s/Str)
+   properties :- (s/maybe {s/Str s/Str})] ;; For now stick with str -> str
+  (let [uuid (or uuid (kitchensink/uuid))]
+    (mq/send-message! mq-connection mq-endpoint raw-command
+                      ;; Until/unless we require that all callers
+                      ;; include received, etc.
+                      (merge {"received" (kitchensink/timestamp) "id" uuid}
+                             properties))
+    uuid))
+
+(defn-validated ^:private do-enqueue-command :- s/Str
+  "Submits command to the mq-endpoint of mq-connection and returns
+  its id. Annotates the command via annotate-command."
+  [mq-connection :- mq/connection-schema
+   mq-endpoint :- s/Str
+   command :- s/Str
+   version :- s/Int
+   payload
+   uuid :- (s/maybe s/Str)
+   properties]
+  (let [command-map {:command command
+                     :version version
+                     :payload payload}]
+    (do-enqueue-raw-command mq-connection mq-endpoint
+                            (json/generate-string command-map)
+                            uuid
+                            properties)))
 
 ;; Catalog replacement
 
 (defn replace-catalog*
-  [{:keys [payload annotations version]} {:keys [db catalog-hash-debug-dir]}]
-  (let [received-timestamp (:received annotations)
-        catalog (upon-error-throw-fatality (cat/parse-catalog payload version received-timestamp))
-        certname (:certname catalog)
-        id (:id annotations)
-        producer-timestamp (:producer_timestamp catalog)]
+  [{:keys [payload annotations version]} db]
+  (let [{id :id received-timestamp :received} annotations
+        {producer-timestamp :producer_timestamp certname :certname :as catalog} payload]
     (jdbc/with-transacted-connection' db :repeatable-read
       (scf-storage/maybe-activate-node! certname producer-timestamp)
-      ;; Only store a catalog if its producer_timestamp is <= the existing catalog's.
-      (if-not (scf-storage/catalog-newer-than? certname producer-timestamp)
-        (scf-storage/replace-catalog! catalog received-timestamp catalog-hash-debug-dir)
-        (log/warnf "Not replacing catalog for certname %s because local data is newer." certname)))
+      (scf-storage/store-catalog! catalog received-timestamp))
     (log/infof "[%s] [%s] %s" id (command-names :replace-catalog) certname)))
 
-(defn warn-deprecated
-  "Logs a deprecation warning message for the given `command` and `version`"
-  [version command]
-  (log/warnf "command '%s' version %s is deprecated, use the latest version" command version))
-
-(defmethod process-command! [(command-names :replace-catalog) 4]
-  [command options]
-  (replace-catalog* command options))
-
-(defmethod process-command! [(command-names :replace-catalog) 5]
-  [command options]
-  (replace-catalog* command options))
-
-(defmethod process-command! [(command-names :replace-catalog) 6]
-  [command options]
-  (replace-catalog* command options))
-
-(defmethod process-command! [(command-names :replace-catalog) 7]
-  [command options]
-  (replace-catalog* command options))
+(defn replace-catalog [{:keys [payload annotations version] :as command} db]
+  (let [{received-timestamp :received} annotations
+        validated-payload (upon-error-throw-fatality
+                           (cat/parse-catalog payload version received-timestamp))]
+    (-> command
+        (assoc :payload validated-payload)
+        (replace-catalog* db))))
 
 ;; Fact replacement
 
-(defmethod process-command! [(command-names :replace-facts) 4]
-  [{:keys [payload annotations]} {:keys [db]}]
-  (let [{:keys [certname values] :as fact-data} payload
-        id        (:id annotations)
-        received-timestamp (:received annotations)
-        fact-data (-> fact-data
-                      (update-in [:values] utils/stringify-keys)
-                      (update-in [:producer_timestamp] to-timestamp)
-                      (assoc :timestamp received-timestamp)
-                      upon-error-throw-fatality)
+(defn replace-facts*
+  [{:keys [payload annotations version]} db]
+  (let [{:keys [id]} annotations
+        {:keys [certname values] :as fact-data} payload
         producer-timestamp (:producer_timestamp fact-data)]
     (jdbc/with-transacted-connection' db :repeatable-read
       (scf-storage/maybe-activate-node! certname producer-timestamp)
       (scf-storage/replace-facts! fact-data))
     (log/infof "[%s] [%s] %s" id (command-names :replace-facts) certname)))
 
-(defmethod process-command! [(command-names :replace-facts) 3]
-  [command config]
-  (-> command
-      fact/v3-wire->v4-wire
-      (process-command! config)))
-
-(defmethod process-command! [(command-names :replace-facts) 2]
-  [command config]
-  (let [received-time (get-in command [:annotations :received])]
+(defn replace-facts [{:keys [payload version annotations] :as command} db]
+  (let [{received-timestamp :received} annotations
+        latest-version-of-payload (case version
+                                    2 (fact/wire-v2->wire-v4 payload received-timestamp)
+                                    3 (fact/wire-v3->wire-v4 payload)
+                                    payload)
+        validated-payload (upon-error-throw-fatality
+                           (-> latest-version-of-payload
+                               (update :values utils/stringify-keys)
+                               (update :producer_timestamp to-timestamp)
+                               (assoc :timestamp received-timestamp)))]
     (-> command
-        (fact/v2-wire->v4-wire received-time)
-        (process-command! config))))
+        (assoc :payload validated-payload)
+        (replace-facts* db))))
 
 ;; Node deactivation
-(defmethod process-command! [(command-names :deactivate-node) 1]
-  [command config]
-  (-> command
-      (assoc :version 2)
-      (update :payload #(upon-error-throw-fatality (json/parse-string % true)))
-      (process-command! config)))
 
-(defmethod process-command! [(command-names :deactivate-node) 2]
-  [command config]
-  (-> command
-      (assoc :version 3)
-      (update :payload #(hash-map :certname %))
-      (process-command! config)))
+(defn deactivate-node-wire-v2->wire-3 [deactive-node]
+  {:certname deactive-node})
 
-(defmethod process-command! [(command-names :deactivate-node) 3]
-  [{:keys [payload annotations]} {:keys [db]}]
+(defn deactivate-node-wire-v1->wire-3 [deactive-node]
+  (-> deactive-node
+      (json/parse-string true)
+      upon-error-throw-fatality
+      deactivate-node-wire-v2->wire-3))
+
+(defn deactivate-node*
+  [{:keys [payload annotations]} db]
   (let [certname (:certname payload)
         producer-timestamp (to-timestamp (:producer_timestamp payload (now)))
-        id  (:id annotations)]
+        id (:id annotations)]
     (jdbc/with-transacted-connection db
       (when-not (scf-storage/certname-exists? certname)
         (scf-storage/add-certname! certname))
       (scf-storage/deactivate-node! certname producer-timestamp))
     (log/infof "[%s] [%s] %s" id (command-names :deactivate-node) certname)))
 
+(defn deactivate-node [{:keys [payload version] :as command} db]
+  (-> command
+      (assoc :payload (case version
+                        1 (deactivate-node-wire-v1->wire-3 payload)
+                        2 (deactivate-node-wire-v2->wire-3 payload)
+                        payload))
+      (deactivate-node* db)))
+
 ;; Report submission
 
 (defn store-report*
-  [version db {:keys [payload annotations]}]
+  [{:keys [payload annotations]} db]
   (let [{id :id received-timestamp :received} annotations
-        {:keys [certname puppet_version] :as report}
-        (->> payload
-             (s/validate report/report-wireformat-schema)
-             upon-error-throw-fatality)
+        {:keys [certname puppet_version] :as report} payload
         producer-timestamp (to-timestamp (:producer_timestamp payload (now)))]
     (jdbc/with-transacted-connection db
       (scf-storage/maybe-activate-node! certname producer-timestamp)
@@ -280,38 +324,57 @@
                id (command-names :store-report)
                puppet_version certname)))
 
-(defmethod process-command! [(command-names :store-report) 3]
-  [command {:keys [db]}]
-  (store-report* 6 db
-                 (let [received-time (get-in command [:annotations :received])]
-                   (update command :payload report/wire-v3->wire-v6 received-time))))
+(defn store-report [{:keys [payload version annotations] :as command} db]
+  (let [{received-timestamp :received} annotations
+        latest-version-of-payload (case version
+                                    3 (report/wire-v3->wire-v7 payload received-timestamp)
+                                    4 (report/wire-v4->wire-v7 payload received-timestamp)
+                                    5 (report/wire-v5->wire-v7 payload)
+                                    6 (report/wire-v6->wire-v7 payload)
+                                    payload)
+        validated-payload (upon-error-throw-fatality
+                           (s/validate report/report-wireformat-schema latest-version-of-payload))]
+    (-> command
+        (assoc :payload validated-payload)
+        (store-report* db))))
 
-(defmethod process-command! [(command-names :store-report) 4]
-  [command {:keys [db]}]
-  (store-report* 6 db
-                 (let [received-time (get-in command [:annotations :received])]
-                   (update command :payload report/wire-v4->wire-v6 received-time))))
-
-(defmethod process-command! [(command-names :store-report) 5]
-  [command {:keys [db]}]
-  (store-report* 6 db
-                 (update command :payload report/wire-v5->wire-v6)))
-
-(defmethod process-command! [(command-names :store-report) 6]
-  [command {:keys [db]}]
-  (store-report* 6 db command))
+;; ## Command processors
 
 (def supported-command?
   (comp (kitchensink/valset command-names) :command))
 
+(defn supported-version? [command version]
+  (contains? (get supported-command-versions command #{}) version))
+
+(defn supported-command-version? [command-name [received-command-name received-version]]
+  (boolean
+   (and (= command-name received-command-name)
+        (supported-version? command-name received-version))))
+
+(defn process-command!
+  "Takes a command object and processes it to completion. Dispatch is
+  based on the command's name and version information"
+  [{command-name :command version :version :as command} db]
+  (condp supported-command-version? [command-name version]
+    "replace catalog" (replace-catalog command db)
+    "replace facts" (replace-facts command db)
+    "store report" (store-report command db)
+    "deactivate node" (deactivate-node command db)))
+
+(defn warn-deprecated
+  "Logs a deprecation warning message for the given `command` and `version`"
+  [version command]
+  (log/warnf "command '%s' version %s is deprecated, use the latest version" command version))
+
 (defprotocol PuppetDBCommandDispatcher
   (enqueue-command
     [this command version payload]
     [this command version payload uuid]
+    [this command version payload uuid properties]
     "Annotates the command via annotate-command, submits it for
     processing, and then returns its unique id.")
 
-  (enqueue-raw-command [this raw-command uuid]
+  (enqueue-raw-command [this raw-command uuid properties]
     "Submits the raw-command for processing and returns the command's
     unique id.")
 
@@ -332,12 +395,15 @@
      id (command uuid)."))
 
 (defn make-cmd-processed-message [cmd ex]
-  (merge {:id (-> cmd :annotations :id)}
+  (merge {:id (-> cmd :annotations :id)
+          :command (:command cmd)
+          :version (:version cmd)
+          :producer-timestamp (-> cmd :payload :producer_timestamp)}
          (when ex {:exception ex})))
 
-(defn process-command-and-respond! [cmd config response-pub-chan stats-atom]
+(defn process-command-and-respond! [cmd db response-pub-chan stats-atom]
   (try
-    (let [result (process-command! cmd config)]
+    (let [result (process-command! cmd db)]
       (swap! stats-atom update :executed-commands inc)
       (async/>!! response-pub-chan
                  (make-cmd-processed-message cmd nil))
@@ -366,16 +432,14 @@
              :response-pub (async/pub response-chan-for-pub :id))))
 
   (start [this context]
-    (let [{:keys [scf-write-db catalog-hash-debug-dir]} (shared-globals)
-          db-cfg {:db scf-write-db
-                  :catalog-hash-debug-dir catalog-hash-debug-dir}
+    (let [{:keys [scf-write-db]} (shared-globals)
           {:keys [response-chan response-pub]} context
           factory (-> (conf/mq-broker-url (get-config))
                       (mq/activemq-connection-factory))
           connection (.createConnection factory)]
       (register-listener
        supported-command?
-       #(process-command-and-respond! % db-cfg response-chan (:stats context)))
+       #(process-command-and-respond! % scf-write-db response-chan (:stats context)))
       (assoc context
              :factory factory
              :connection connection)))
@@ -397,21 +461,25 @@
     (enqueue-command this command version payload nil))
 
   (enqueue-command [this command version payload uuid]
+    (enqueue-command this command version payload uuid nil))
+
+  (enqueue-command [this command version payload uuid properties]
     (let [config (get-config)
           connection (:connection (service-context this))
           endpoint (get-in config [:command-processing :mq :endpoint])
           command (if (string? command) command (command-names command))
           result (do-enqueue-command connection endpoint
-                                     command version payload uuid)]
+                                      command version payload uuid properties)]
       ;; Obviously assumes that if do-* doesn't throw, msg is in
       (swap! (:stats (service-context this)) update :received-commands inc)
       result))
 
-  (enqueue-raw-command [this raw-command uuid]
+  (enqueue-raw-command [this raw-command uuid properties]
     (let [config (get-config)
           connection (:connection (service-context this))
           endpoint (get-in config [:command-processing :mq :endpoint])
-          result (do-enqueue-raw-command connection endpoint raw-command uuid)]
+          result (do-enqueue-raw-command connection endpoint
+                                          raw-command uuid properties)]
       ;; Obviously assumes that if do-* doesn't throw, msg is in
       (swap! (:stats (service-context this)) update :received-commands inc)
       result))
