@@ -54,8 +54,9 @@
      :conn-max-age (pls/defaulted-maybe s/Int 60)
      :conn-keep-alive (pls/defaulted-maybe s/Int 45)
      :conn-lifetime (s/maybe s/Int)
-     :classname (s/maybe String)
-     :subprotocol (s/maybe String)
+     :maximum-pool-size (pls/defaulted-maybe s/Int 10)
+     :classname (pls/defaulted-maybe String "org.postgresql.Driver")
+     :subprotocol (pls/defaulted-maybe String "postgresql")
      :subname (s/maybe String)
      :username String
      :user String
@@ -68,7 +69,7 @@
      :stats (pls/defaulted-maybe String "true")
      :log-statements (pls/defaulted-maybe String "true")
      :statements-cache-size (pls/defaulted-maybe s/Int 0)
-     :connection-timeout (pls/defaulted-maybe s/Int 500)}))
+     :connection-timeout (pls/defaulted-maybe s/Int 3000)}))
 
 (def write-database-config-in
   "Includes the common database config params, also the write-db specific ones"
@@ -78,8 +79,7 @@
             :dlo-compression-interval s/Int
             :report-ttl (pls/defaulted-maybe String "14d")
             :node-purge-ttl (pls/defaulted-maybe String "0s")
-            :node-ttl (pls/defaulted-maybe String "0s")
-            :connection-timeout (pls/defaulted-maybe s/Int 1000)})))
+            :node-ttl (pls/defaulted-maybe String "0s")})))
 
 (def database-config-out
   "Schema for parsed/processed database config"
@@ -97,6 +97,7 @@
    :log-statements Boolean
    :statements-cache-size s/Int
    :connection-timeout s/Int
+   :maximum-pool-size s/Int
    (s/optional-key :conn-lifetime) (s/maybe Minutes)
    (s/optional-key :username) String
    (s/optional-key :user) String
@@ -154,7 +155,7 @@
    :threads s/Int
    :max-frame-size s/Int
    :max-command-size s/Int
-   :reject-large-commands s/Bool
+   :reject-large-commands Boolean
    (s/optional-key :store-usage) s/Int
    (s/optional-key :temp-usage) s/Int})
 
@@ -162,46 +163,39 @@
   "Schema for validating the incoming [puppetdb] block"
   (all-optional
     {:certificate-whitelist s/Str
+     ;; The `historical-catalogs-limit` setting is only used by `pe-puppetdb`
+     :historical-catalogs-limit (pls/defaulted-maybe s/Int 3)
      :disable-update-checking (pls/defaulted-maybe String "false")}))
 
 (def puppetdb-config-out
   "Schema for validating the parsed/processed [puppetdb] block"
   {(s/optional-key :certificate-whitelist) s/Str
+   :historical-catalogs-limit s/Int
    :disable-update-checking Boolean})
 
+(def developer-config-in
+  (all-optional
+    {:pretty-print (pls/defaulted-maybe String "false")}))
+
+(def developer-config-out
+  {:pretty-print Boolean})
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Database config
 
-(defn hsql-default-connection
-  "Returns a map of default, file-backed, HyperSQL connection information"
-  [vardir]
-  {:classname   "org.hsqldb.jdbcDriver"
-   :subprotocol "hsqldb"
-   :subname     (format "file:%s;hsqldb.tx=mvcc;sql.syntax_pgs=true" (io/file vardir "db"))})
-
 (defn validate-db-settings
   "Throws a {:type ::cli-error :message m} exception
   describing the required additions if the [database] configuration
   doesn't specify classname, subprotocol and subname, all of which are
   now required."
-  [{global :global db-config :database :or {db-config {}} :as config}]
-  ;; If the user has none of these settings, then they've either been
-  ;; relying on the old HSQLDB default, or they just have no settings.
-  ;; Testing not-any? is fine because we catch partial configurations
-  ;; elsewhere.
-  (when (not-any? db-config [:classname :subprotocol :subname])
-    (let [default (hsql-default-connection (:vardir global))
-          setting-needed #(when-not (% db-config)
-                            (format "\n  %s = %s" (name %) (% default)))
-          msg (str "database configuration is now required."
-                   " Previously HSQLDB was the default, but support has been deprecated."
-                   " If you were relying on that default,"
-                   " please add this to the [database] section of your config file:"
-                   (setting-needed :classname)
-                   (setting-needed :subprotocol)
-                   (setting-needed :subname))]
-      (throw+ {:type ::cli-error
-               :message msg})))
+  [{db-config :database :or {db-config {}} :as config}]
+  (when (str/blank? (:subname db-config))
+    (throw+
+     {:type ::cli-error
+      :message
+      (str "PuppetDB requires PostgreSQL."
+           "  The [database] section must contain an appropriate"
+           " \"//host:port/database\" subname setting.")}))
   config)
 
 (defn convert-section-config
@@ -234,9 +228,11 @@
 (defn configure-puppetdb
   "Validates the [puppetdb] section of the config"
   [{:keys [puppetdb] :as config :or {puppetdb {}}}]
-  (->> puppetdb
-       (s/validate puppetdb-config-in)
-       (assoc config :puppetdb)))
+  (configure-section config :puppetdb puppetdb-config-in puppetdb-config-out))
+
+(defn configure-developer
+  [{:keys [developer] :as config :or {developer {}}}]
+  (configure-section config :developer developer-config-in developer-config-out))
 
 (defn configure-command-processing
   [config]
@@ -260,7 +256,10 @@
   "Checks that `vardir` is specified, exists, and is writeable, throwing
   appropriate exceptions if any condition is unmet."
   [config]
-  (let [vardir (io/file (get-in config [:global :vardir]))]
+  (let [vardir (some-> (get-in config [:global :vardir])
+                       io/file
+                       fs/absolute-path
+                       fs/file)]
     (cond
      (nil? vardir)
      (throw (IllegalArgumentException.
@@ -285,47 +284,6 @@
      :else
      config)))
 
-(defn catalog-debug-path
-  "Given a `config` create the path to the directory directory to store
-   catalog debug info."
-  [{{vardir :vardir} :global}]
-  {:pre [(not (str/blank? vardir))]}
-  (fs/file vardir "debug" "catalog-hashes"))
-
-(defn create-catalog-debug-dir
-  "Attempt to crate the catalog debug directory at `path`. Failing to create the
-   directory only causes a warning as not having this directory shouldn't cause
-   PuppetDB to crash on startup."
-  [path]
-  (try
-    (do (fs/mkdirs path) (str path))
-    (catch SecurityException e
-      (log/warnf e
-                (str "catalog-hash-conflig-debugging was enabled, "
-                     "but PuppetDB was not able to create a directory at %s")
-                path))))
-
-(def ^{:doc "Create the directory for catalog debug info if it does not already
-              exist, returning the path if successful (or it already exists)"}
-  ensure-catalog-debug-dir
-  (comp create-catalog-debug-dir catalog-debug-path))
-
-(defn configure-catalog-debugging
-  "When [global] contains catalog-hash-conflict-debugging=true, assoc into the config the directory
-   to store the debugging, if not return the config unmodified."
-  [config]
-  (if-let [debug-dir (and (kitchensink/true-str? (get-in config [:global :catalog-hash-conflict-debugging]))
-                          (ensure-catalog-debug-dir config))]
-    (do
-      (log/warn (str "Global config catalog-hash-conflict-debugging set to true. "
-                     "This is intended to troubleshoot catalog duplication issues and "
-                     "not for enabling in production long term.  See the PuppetDB docs "
-                     "for more information on this setting."))
-      (log/warn (str "The config item 'catalog-hash-conflict-debugging' is "
-                     "deprecated and will be retired in a future release."))
-      (assoc-in config [:global :catalog-hash-debug-dir] debug-dir))
-    config))
-
 (defn normalize-product-name
   "Checks that `product-name` is specified as a legal value, throwing an
   exception if not. Returns `product-name` if it's okay."
@@ -348,8 +306,19 @@
                (utils/assoc-when :update-server "http://updates.puppetlabs.com/check-for-updates"))))
 
 (defn warn-retirements
-  "Warn a user they are using the old [repl] block, instead of [nrepl]."
+  "Warns about configuration retirements.  Abruptly exits the entire
+  process if a [global] url-prefix is found."
   [config-data]
+  (doseq [param [:classname :subprotocol]]
+    (when (get-in config-data [:database param])
+      (utils/println-err
+       (format "The [database] %s setting has been retired and will be ignored."
+               (name param)))))
+  (when (get-in config-data [:global :catalog-hash-conflict-debugging])
+    (utils/println-err (str "The configuration item `catalog-hash-conflict-debugging`"
+                            " in the [global] section is retired,"
+                            " please remove this item from your config."
+                            " Consult the documentation for more details.")))
   (when (get-in config-data [:repl])
     (utils/println-err (str "The configuration block [repl] is now retired and will be ignored."
                             " Use [nrepl] instead. Consult the documentation for more details.")))
@@ -358,12 +327,15 @@
                             " please remove this item from your config."
                             " PuppetDB has a non-configurable context route of `/pdb`."
                             " Consult the documentation for more details."))
-    (System/exit 1))
+    (flush)
+    (binding [*out* *err*] (flush))
+    (System/exit 1)) ; cf. PDB-2053
   config-data)
 
 (defn add-web-routing-service-config
   [config-data]
   (let [default-web-router-service {:puppetlabs.puppetdb.metrics/metrics-service "/metrics"
+                                    :puppetlabs.trapperkeeper.services.status.status-service/status-service "/status"
                                     :puppetlabs.puppetdb.pdb-routing/pdb-routing-service "/pdb"}
         bootstrap-cfg (-> (find-bootstrap-config config-data)
                           slurp
@@ -408,9 +380,9 @@
   [config]
   (-> config
       configure-globals
+      configure-developer
       validate-vardir
       convert-config
-      configure-catalog-debugging
       add-mq-defaults))
 
 (defn foss? [config]
