@@ -190,9 +190,11 @@
 (defn stream-json
   "Serializes the supplied sequence to `buffer`, which is a `Writer`
   object."
-  [coll buffer]
+  [coll buffer pretty-print]
   {:pre [(instance? Writer buffer)]}
-  (json/generate-pretty-stream coll buffer))
+  (if pretty-print
+    (json/generate-pretty-stream coll buffer)
+    (json/generate-stream coll buffer)))
 
 (defmacro streamed-response
   "Evaluates `body` in a thread, with a local variable (`writer-var`)
@@ -281,3 +283,18 @@
   "Produce a json 400 response with an :error key holding message."
   [message]
   (json-response {:error message} status-bad-request))
+
+(defn deprecated-app
+  "Add an X-Deprecation warning for deprecated endpoints"
+  [app msg request]
+  (let [result (app request)]
+    (log/warn msg)
+    (rr/header result "X-Deprecation" msg)))
+
+(defn experimental-warning
+  "Add a Warning: header for experimental endpoints"
+  [app msg]
+  (fn [request]
+    (let [result (app request)]
+      (log/warn msg)
+      (rr/header result "Warning" msg))))
