@@ -1,6 +1,6 @@
 (ns puppetlabs.puppetdb.jdbc
   "Database utilities"
-  (:import (com.jolbox.bonecp BoneCPDataSource BoneCPConfig)
+  (:import (com.zaxxer.hikari HikariDataSource HikariConfig)
            (java.util.concurrent TimeUnit))
   (:require [clojure.java.jdbc :as sql]
             [clojure.string :as string]
@@ -8,7 +8,7 @@
             [puppetlabs.kitchensink.core :as kitchensink]
             [clojure.string :as str]
             [puppetlabs.puppetdb.time :as pl-time]
-            [puppetlabs.puppetdb.jdbc.internal :refer :all]
+            [puppetlabs.puppetdb.jdbc.internal :refer [limit-result-set!]]
             [puppetlabs.puppetdb.schema :as pls]
             [schema.core :as s]
             [clojure.math.numeric-tower :as math]))
@@ -44,6 +44,16 @@
   [transaction? & remainder]
   (apply sql/db-do-prepared *db* transaction? remainder))
 
+(defn do-commands-outside-txn [& commands]
+  (let [conn (:connection *db*)
+        orig (.getAutoCommit conn)]
+    (.setAutoCommit conn true)
+    (try
+      (doseq [c commands]
+        (with-open [s (.createStatement conn)]
+          (.execute s c)))
+      (finally (.setAutoCommit conn orig)))))
+
 (defn insert!
   "Calls clojure.jdbc/insert! after adding (jdbc/db) as the first argument."
   {:arglists '([table row-map :transaction? true :entities identity]
@@ -102,8 +112,7 @@
   [q]
   (and
    (vector? q)
-   (string? (first q))
-   (every? (complement coll?) (rest q))))
+   (string? (first q))))
 
 (def valid-results-query-schema
   "Schema type for compiled query-eng queries"
@@ -231,7 +240,7 @@
    In most cases a base of 2 is sufficient, but you can adjust this to create
    tighter or looser sleep cycles."
   [current-attempt :- s/Int
-   base :- (s/either s/Int Double)]
+   base :- (s/cond-pre s/Int Double)]
   (let [sleep-ms (-> (math/expt base current-attempt)
                      (- 1)
                      (* 1000))]
@@ -274,16 +283,8 @@
                       [(f)]
                       ;; Catch connection errors, and retry for some of them.
                       ;; cf. PostgreSQL docs: Appendix A. PostgreSQL Error Codes
-                      (catch java.sql.SQLException e
-                        ;; This includes org.postgresql.util.PSQLException
-                        (let [sqlstate (.getSQLState e)]
-                          (case sqlstate
-                            ;; The connection does not exist
-                            "08003" (retry-sql-or-fail r current e)
-                            ;; PostgreSQL was restarted
-                            "57P01" (retry-sql-or-fail r current e)
-                            ;; All other errors are not retried
-                            (throw e)))))]
+                      (catch java.sql.SQLTransientConnectionException e
+                        (retry-sql-or-fail r current e)))]
       (result 0)
       (recur (dec r) (inc current)))))
 
@@ -349,52 +350,57 @@
            (.cancel stmt)
            (throw e)))))))
 
-(defn make-connection-pool
-  "Create a new database connection pool"
-  [{:keys [classname subprotocol subname user username password
-           partition-conn-min partition-conn-max partition-count
-           stats log-statements log-slow-statements statements-cache-size
-           conn-max-age conn-lifetime conn-keep-alive read-only?
-           connection-timeout pool-availability-threshold]
-    :as db}]
-  (let [;; Load the database driver class explicitly, to avoid jar load ordering
-        ;; issues.
-        _ (Class/forName classname)
-        log-slow-statements-duration (pl-time/to-seconds log-slow-statements)
-        config          (doto (new BoneCPConfig)
-                          (.setDefaultAutoCommit false)
-                          (.setLazyInit true)
-                          (.setMinConnectionsPerPartition partition-conn-min)
-                          (.setMaxConnectionsPerPartition partition-conn-max)
-                          (.setPartitionCount partition-count)
-                          (.setConnectionTestStatement "begin; select 1; commit;")
-                          (.setStatisticsEnabled stats)
-                          (.setIdleMaxAgeInMinutes (pl-time/to-minutes conn-max-age))
-                          (.setIdleConnectionTestPeriodInMinutes (pl-time/to-minutes conn-keep-alive))
-                          ;; paste the URL back together from parts.
-                          (.setJdbcUrl (str "jdbc:" subprotocol ":" subname))
-                          (.setConnectionHook (connection-hook log-statements log-slow-statements-duration))
-                          (.setStatementsCacheSize statements-cache-size)
-                          (.setDefaultReadOnly read-only?)
-                          (.setConnectionTimeoutInMs connection-timeout))
-        user (or user username)]
-    ;; configurable without default
-    (some->> user str (.setUsername config))
-    (some->> password str (.setPassword config))
-    (some->> pool-availability-threshold (.setPoolAvailabilityThreshold config))
-    (some->> log-statements (.setLogStatementsEnabled config))
-    (when conn-lifetime (.setMaxConnectionAge config (pl-time/to-minutes conn-lifetime) TimeUnit/MINUTES))
+(defn ^:dynamic enable-jmx
+  "This function exists to enable starting multiple PuppetDB instances
+  inside a single JVM. Starting up a second instance results in a
+  collision exception between JMX beans from the two
+  instances. Disabling JMX from the broker avoids that issue"
+  [^HikariConfig config metrics-registry]
+  (.setMetricRegistry config metrics-registry))
 
-    (.setQueryExecuteTimeLimit config log-slow-statements-duration (TimeUnit/SECONDS))
-    ;; ...aaand, create the pool.
-    (BoneCPDataSource. config)))
+(defn make-connection-pool
+  "Given a DB spec map containing :subprotocol, :subname, :user, and :password
+  keys, return a pooled DB spec map (one containing just the :datasource key
+  with a pooled DataSource object as the value). The returned pooled DB spec
+  can be passed directly as the first argument to clojure.java.jdbc's
+  functions.
+
+  Times out after 30 seconds and throws org.postgresql.util.PSQLException"
+  ([db-spec] (make-connection-pool db-spec nil))
+  ([{:keys [subprotocol subname
+            user username password
+            connection-timeout
+            conn-max-age
+            conn-lifetime
+            read-only?
+            pool-name]
+     :as db-spec}
+    metrics-registry]
+   (let [conn-lifetime-ms (some-> conn-max-age pl-time/to-millis)
+         conn-max-age-ms (some-> conn-lifetime pl-time/to-millis)
+         config (HikariConfig.)]
+     (doto config
+       (.setJdbcUrl (str "jdbc:" subprotocol ":" subname))
+       (.setAutoCommit false)
+       (.setInitializationFailFast false))
+     (some->> pool-name (.setPoolName config))
+     (some->> connection-timeout (.setConnectionTimeout config))
+     (when (and conn-max-age-ms conn-lifetime-ms (> conn-max-age-ms conn-lifetime-ms))
+       (some->> conn-max-age-ms (.setIdleTimeout config)))
+     (some->> conn-lifetime-ms (.setMaxLifetime config))
+     (some->> read-only? (.setReadOnly config))
+     (some->> (or user username) str (.setUsername config))
+     (some->> password str (.setPassword config))
+     (some->> metrics-registry (enable-jmx config))
+     (HikariDataSource. config))))
 
 (defn pooled-datasource
   "Given a database connection attribute map, return a JDBC datasource
   compatible with clojure.java.jdbc that is backed by a connection
   pool."
-  [options]
-  {:datasource (make-connection-pool options)})
+  ([options] (pooled-datasource options nil))
+  ([options metrics-registry]
+   {:datasource (make-connection-pool options metrics-registry)}))
 
 (defn in-clause
   "Create a prepared statement in clause, with a ? for every item in coll"
