@@ -7,17 +7,25 @@
             [puppetlabs.puppetdb.http :as http]
             [ring.util.response :as rr]
             [ring.util.request :as request]
-            [clojure.string :as s]
+            [clojure.string :as str]
             [clojure.tools.logging :as log]
             [ring.middleware.params :refer [wrap-params]]
             [puppetlabs.puppetdb.query.paging :as paging]
             [clojure.set :as set]
             [pantomime.media :as media]
+            [puppetlabs.puppetdb.metrics.core :as metrics]
             [puppetlabs.puppetdb.jdbc :refer [with-transacted-connection]]
             [metrics.timers :refer [timer time!]]
             [metrics.meters :refer [meter mark!]]
             [clojure.walk :refer [keywordize-keys]]
-            [puppetlabs.puppetdb.utils :as utils]))
+            [puppetlabs.puppetdb.utils :as utils]
+            [bidi.bidi :as bidi]
+            [bidi.ring :as bring]
+            [bidi.schema :as bidi-schema]
+            [puppetlabs.puppetdb.schema :as pls]
+            [schema.core :as s]))
+
+(def handler-schema (s/=> s/Any {s/Any s/Any}))
 
 (defn wrap-with-debug-logging
   "Ring middleware that logs incoming HTTP request URIs (at DEBUG level) as
@@ -94,30 +102,17 @@
 
 (defn wrap-with-globals
   "Ring middleware that adds a :globals attribute to each request that
-  contains a map of the current shared-global settings."
+   contains a map of the current shared-global settings."
   [app get-shared-globals]
   (fn [req]
-    (let [new-req (assoc req :globals (get-shared-globals))]
+    (let [new-req (update req :globals #(merge (get-shared-globals) %))]
       (app new-req))))
 
-(defn wrap-with-paging-options
-  "Ring middleware that will add to each request a :paging-options attribute:
-  a map optionally containing :limit, :offset, and :order_by keys used to
-  implement result paging for the query.  The value for :order_by will be
-  a list of maps, containing information about the fields to order the result
-  by.  Each order_by map contains a key :field, and an optional key :order
-  (whose value may be either 'asc' or 'desc', and defaults to 'asc')."
+(defn wrap-with-illegal-argument-catch
   [app]
-  (fn [{:keys [params] :as req}]
+  (fn [req]
     (try
-      (app (assoc req :paging-options
-                  (-> params
-                      (select-keys ["limit" "offset" "order_by" "include_total"])
-                      keywordize-keys
-                      paging/parse-limit
-                      paging/parse-offset
-                      paging/parse-count
-                      paging/parse-order-by)))
+      (app req)
       (catch IllegalArgumentException e
         (http/error-response e)))))
 
@@ -191,55 +186,9 @@
   (fn [app]
     (verify-accepts-content-type app "application/json")))
 
-(defn verify-checksum
-  "Ring middleware which will verify that the content of the body-string
-  has the checksum specified in the `checksum` parameter. If no checksum is
-  provided, this check will be skipped. If the checksum doesn't match, a 400
-  Bad Request error is returned."
-  [app]
-  (fn [{:keys [params body-string] :as req}]
-    (let [expected-checksum (params "checksum")
-          payload           body-string]
-      (if (and expected-checksum
-               (not= expected-checksum (kitchensink/utf8-string->sha1 payload)))
-        (http/error-response "checksums don't match")
-        (app req)))))
+(def http-metrics-registry (get-in metrics/metrics-registries [:http :registry]))
 
-(defn wrap-with-metrics*
-  "Ring middleware that will tack performance counters for each
-  URL. Arguments are the same as for `wrap-with-metrics`, except:
-
-  `prefix`: string to use as the first component of each generated
-  metric."
-  [app prefix storage normalize-uri]
-  (fn [req]
-    ;; add metric timers for the uri as we service the request.
-    (let [metric-roots (let [s (normalize-uri (:uri req))]
-                         (if (string? s) [s] s))
-          metric-roots (map #(s/replace % #"[:,=]" "_") metric-roots)]
-
-      ;; Create timer objects for each metric the user has requested
-      (doseq [metric-root metric-roots
-              :let [timer-key [:timers metric-root]]
-              :when (not (get-in @storage timer-key))]
-        (swap! storage assoc-in timer-key (timer [prefix metric-root "service-time"])))
-
-      (let [timers (map #(get-in @storage [:timers %]) metric-roots)]
-        (multitime! timers
-                    (let [response  (app req)
-                          status    (:status response)]
-
-                      ;; Create meter objects for each metric the user has
-                      ;; requested
-                      (doseq [metric-root metric-roots
-                              :let [meter-key [:meters metric-root status]]
-                              :when (not (get-in @storage meter-key))]
-                        (swap! storage assoc-in meter-key (meter [prefix metric-root (str status)] "reqs/s"))
-                        (mark! (get-in @storage meter-key)))
-
-                        response))))))
-
-(defmacro wrap-with-metrics
+(defn wrap-with-metrics
   "Ring middleware that will tack performance counters for each URL.
   We track the following metrics per-app, at a top-level
 
@@ -273,32 +222,32 @@
   cannot contain ':', '=', or ',' characters. They will be replaced
   with '_'."
   [app storage normalize-uri]
-  `(let [prefix# ~(str *ns*)]
-    (wrap-with-metrics* ~app prefix# ~storage ~normalize-uri)))
+  (fn [req]
+    ;; add metric timers for the uri as we service the request.
+    (let [metric-roots (let [s (normalize-uri (:uri req))]
+                         (if (string? s) [s] s))
+          metric-roots (map #(str/replace % #"[:,=]" "_") metric-roots)]
 
-(defn payload-to-body-string
-  "Middleware to move the payload from the body or the payload param into the
-  request property `:body-string`.
+      ;; Create timer objects for each metric the user has requested
+      (doseq [metric-root metric-roots
+              :let [timer-key [:timers metric-root]]
+              :when (not (get-in @storage timer-key))]
+        (swap! storage assoc-in timer-key (timer http-metrics-registry [metric-root "service-time"])))
 
-  Usually the body is an InputStream, it is first converted to a string."
-  [app]
-  (fn [{:keys [body params headers] :as req}]
-    (let [content-type (headers "content-type")
-          mediatype (if (nil? content-type) nil
-                        (str (media/base-type content-type)))]
-      (case mediatype
-        "application/x-www-form-urlencoded"
-        (if-let [payload (params "payload")]
-          (app (assoc req :body-string payload))
-          (http/error-response (str "Missing required parameter 'payload'")))
-        "application/json"
-        (let [body-string (request/body-string req)]
-          (if (nil? body-string)
-            (http/error-response (str "Empty body for application/json submission"))
-            (app (assoc req :body-string body-string))))
-        (if-let [payload (params "payload")]
-          (app (assoc req :body-string payload))
-          (http/error-response (str "Missing required parameter 'payload'")))))))
+      (let [timers (map #(get-in @storage [:timers %]) metric-roots)]
+        (multitime! timers
+                    (let [response  (app req)
+                          status    (:status response)]
+
+                      ;; Create meter objects for each metric the user has
+                      ;; requested
+                      (doseq [metric-root metric-roots
+                              :let [meter-key [:meters metric-root status]]
+                              :when (not (get-in @storage meter-key))]
+                        (swap! storage assoc-in meter-key (meter http-metrics-registry [metric-root (str status)]))
+                        (mark! (get-in @storage meter-key)))
+
+                        response))))))
 
 (defn consume-and-close
   "Consume all data from input stream and then close"
@@ -331,9 +280,9 @@
                        length-in-bytes
                        max-command-size)
             (consume-and-close (:body req) length-in-bytes)
-            {:status 413
-             :headers {}
-             :body "Command rejected due to size exceeding max-command-size"})
+            (http/error-response
+             "Command rejected due to size exceeding max-command-size"
+             http/status-entity-too-large))
           (app req)))
       (app req))))
 
@@ -347,18 +296,42 @@
       wrap-with-default-body
       wrap-with-debug-logging))
 
-(defn wrap-with-parent-check
+(defn parent-check
   "Middleware that checks the parent exists before serving the rest of the
    application. This ensures we always return 404's on child paths when the
    parent data is empty."
-  [app version parent id]
-  (fn [{:keys [globals] :as req}]
+  [app version parent route-param-key]
+  (fn [{:keys [globals route-params] :as req}]
     (let [{:keys [scf-read-db url-prefix]} globals]
       ;; There is a race condition here, in particular we open up 1 transaction
       ;; for the parent test, but the rest of the query results are done via the
       ;; streaming query. This can't be solved until we work out a way to
       ;; pass through an existing db handle through to the streamed query thread.
       (if (jdbc/with-transacted-connection scf-read-db
-            (qe/object-exists? parent id))
+            (qe/object-exists? parent (get route-params route-param-key)))
         (app req)
-        (http/json-response {:error (str "No information is known about " (name parent) " " id)} http/status-not-found)))))
+        (http/json-response {:error (str "No information is known about " (name parent) " " (get route-params route-param-key))} http/status-not-found)))))
+
+(pls/defn-validated url-decode :- s/Str
+  [x :- s/Str]
+  (java.net.URLDecoder/decode x))
+
+(pls/defn-validated make-pdb-handler :- handler-schema
+  "Similar to `bidi.ring/make-handler` but does not merge route-params
+  into the regular parameter map. Currently route-params causes
+  validation errors with merged in with parameters. Parameter names
+  are currently strings and validated against an expected list. Route
+  params are merged in a keywords."
+  ([route :- bidi-schema/RoutePair]
+   (make-pdb-handler route identity))
+  ([route :- bidi-schema/RoutePair
+    handler-fn :- handler-schema]
+   (fn [{:keys [uri path-info] :as req}]
+     (let [path (or path-info uri)
+           {:keys [handler route-params] :as match-context} (bidi/match-route* route path req)]
+       (when handler
+         (bring/request
+          (handler-fn handler)
+          (update req :route-params merge (kitchensink/mapvals url-decode route-params))
+          (apply dissoc match-context :handler (keys req))))))))
+
