@@ -50,53 +50,52 @@
 (defn with-latest-events
   "CTE to wrap unioned queries when distinct_resources is used"
   [query]
-  (let [collate (if (sutils/postgres?) "COLLATE \"C\"" "")]
-    (format
-      "WITH latest_events AS
-       (SELECT certname,
-               configuration_version,
-               start_time as run_start_time,
-               end_time as run_end_time,
-               receive_time as report_receive_time,
-               hash,
-               status,
-               distinct_events.latest_timestamp AS timestamp,
-               distinct_events.resource_type AS resource_type,
-               distinct_events.resource_title AS resource_title,
-               distinct_events.property as property,
-               new_value,
-               old_value,
-               message,
-               file,
-               line,
-               containment_path,
-               containing_class,
-               environment
-       FROM
-       (SELECT certname_id,
-               resource_type %s AS resource_type,
-               resource_title %s AS resource_title,
-               property %s AS property,
-               MAX(resource_events.timestamp) AS latest_timestamp
-       FROM resource_events
-       WHERE resource_events.timestamp >= ?
-             AND resource_events.timestamp <= ?
-       GROUP BY certname_id,
-                resource_type %s,
-                resource_title %s,
-                property %s) distinct_events
-       INNER JOIN resource_events
-       ON resource_events.resource_type = distinct_events.resource_type
-          AND resource_events.resource_title = distinct_events.resource_title
-          AND ((resource_events.property = distinct_events.property) OR
-               (resource_events.property IS NULL
-                AND distinct_events.property IS NULL))
-       AND resource_events.timestamp = distinct_events.latest_timestamp
-       AND resource_events.certname_id = distinct_events.certname_id
-       INNER JOIN reports ON resource_events.report_id = reports.id
-       LEFT OUTER JOIN environments ON reports.environment_id = environments.id)
-
-       %s" collate collate collate collate collate collate query)))
+  (str
+   "WITH latest_events AS
+      (SELECT certname,
+              configuration_version,
+              start_time as run_start_time,
+              end_time as run_end_time,
+              receive_time as report_receive_time,
+              hash,
+              status,
+              distinct_events.latest_timestamp AS timestamp,
+              distinct_events.resource_type AS resource_type,
+              distinct_events.resource_title AS resource_title,
+              distinct_events.property as property,
+              new_value,
+              old_value,
+              message,
+              file,
+              line,
+              containment_path,
+              containing_class,
+              environment
+      FROM
+      (SELECT certname_id,
+              resource_type COLLATE \"C\" AS resource_type,
+              resource_title COLLATE \"C\" AS resource_title,
+              property COLLATE \"C\" AS property,
+              MAX(resource_events.timestamp) AS latest_timestamp
+      FROM resource_events
+      WHERE resource_events.timestamp >= ?
+            AND resource_events.timestamp <= ?
+      GROUP BY certname_id,
+               resource_type COLLATE \"C\",
+               resource_title COLLATE \"C\",
+               property COLLATE \"C\") distinct_events
+      INNER JOIN resource_events
+      ON resource_events.resource_type = distinct_events.resource_type
+         AND resource_events.resource_title = distinct_events.resource_title
+         AND ((resource_events.property = distinct_events.property) OR
+              (resource_events.property IS NULL
+               AND distinct_events.property IS NULL))
+      AND resource_events.timestamp = distinct_events.latest_timestamp
+      AND resource_events.certname_id = distinct_events.certname_id
+      INNER JOIN reports ON resource_events.report_id = reports.id
+      LEFT OUTER JOIN environments
+        ON reports.environment_id = environments.id) "
+   query))
 
 (defn distinct-select
   "Build the SELECT statement that we use in the `distinct-resources` case (where
