@@ -33,13 +33,13 @@
             [clojure.data :as data]
             [puppetlabs.puppetdb.scf.hash :as shash]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
-            [puppetlabs.puppetdb.scf.hash-debug :as hashdbg]
             [schema.core :as s]
             [puppetlabs.puppetdb.schema :as pls :refer [defn-validated]]
             [puppetlabs.puppetdb.utils :as utils]
             [clj-time.core :refer [now]]
+            [puppetlabs.puppetdb.metrics.core :as metrics]
             [metrics.counters :refer [counter inc! value]]
-            [metrics.gauges :refer [gauge]]
+            [metrics.gauges :refer [gauge-fn]]
             [metrics.histograms :refer [histogram update!]]
             [metrics.timers :refer [timer time!]]
             [puppetlabs.puppetdb.jdbc :refer [query-to-vec]]
@@ -54,7 +54,7 @@
   {:type String
    :title String})
 
-(def json-primitive-schema (s/either String Number Boolean))
+(def json-primitive-schema (s/cond-pre String Number Boolean))
 
 ;; the maximum number of parameters pl-jdbc will admit in a prepared statement
 ;; is 32767. delete-pending-value-id-orphans will create a prepared statement
@@ -109,8 +109,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Metrics
 
-;; This is pinned to the old namespace for backwards compatibility
-(def ns-str "puppetlabs.puppetdb.scf.storage")
+(def storage-metrics-registry (get-in metrics/metrics-registries [:storage :registry]))
 
 ;; ## Performance metrics
 ;;
@@ -168,35 +167,35 @@
 ;;
 (def performance-metrics
   {
-   :add-resources      (timer [ns-str "default" "add-resources"])
-   :add-edges          (timer [ns-str "default" "add-edges"])
+   :add-resources      (timer storage-metrics-registry ["add-resources"])
+   :add-edges          (timer storage-metrics-registry ["add-edges"])
 
-   :resource-hashes    (timer [ns-str "default" "resource-hashes"])
-   :catalog-hash       (timer [ns-str "default" "catalog-hash"])
-   :add-new-catalog    (timer [ns-str "default" "new-catalog-time"])
-   :catalog-hash-match (timer [ns-str "default" "catalog-hash-match-time"])
-   :catalog-hash-miss  (timer [ns-str "default" "catalog-hash-miss-time"])
-   :replace-catalog    (timer [ns-str "default" "replace-catalog-time"])
+   :resource-hashes    (timer storage-metrics-registry ["resource-hashes"])
+   :catalog-hash       (timer storage-metrics-registry ["catalog-hash"])
+   :add-new-catalog    (timer storage-metrics-registry ["new-catalog-time"])
+   :catalog-hash-match (timer storage-metrics-registry ["catalog-hash-match-time"])
+   :catalog-hash-miss  (timer storage-metrics-registry ["catalog-hash-miss-time"])
+   :replace-catalog    (timer storage-metrics-registry ["replace-catalog-time"])
 
-   :gc                 (timer [ns-str "default" "gc-time"])
-   :gc-catalogs        (timer [ns-str "default" "gc-catalogs-time"])
-   :gc-params          (timer [ns-str "default" "gc-params-time"])
-   :gc-environments    (timer [ns-str "default" "gc-environments-time"])
-   :gc-report-statuses (timer [ns-str "default" "gc-report-statuses"])
-   :gc-fact-paths  (timer  [ns-str "default" "gc-fact-paths"])
+   :gc                 (timer storage-metrics-registry ["gc-time"])
+   :gc-catalogs        (timer storage-metrics-registry ["gc-catalogs-time"])
+   :gc-params          (timer storage-metrics-registry ["gc-params-time"])
+   :gc-environments    (timer storage-metrics-registry ["gc-environments-time"])
+   :gc-report-statuses (timer storage-metrics-registry ["gc-report-statuses"])
+   :gc-fact-paths  (timer storage-metrics-registry ["gc-fact-paths"])
 
-   :updated-catalog    (counter [ns-str "default" "new-catalogs"])
-   :duplicate-catalog  (counter [ns-str "default" "duplicate-catalogs"])
-   :duplicate-pct      (gauge [ns-str "default" "duplicate-pct"]
-                              (let [dupes (value (:duplicate-catalog performance-metrics))
-                                    new   (value (:updated-catalog performance-metrics))]
-                                (float (kitchensink/quotient dupes (+ dupes new)))))
-   :catalog-volatility (histogram [ns-str "default" "catalog-volitilty"])
+   :updated-catalog    (counter storage-metrics-registry ["new-catalogs"])
+   :duplicate-catalog  (counter storage-metrics-registry ["duplicate-catalogs"])
+   :duplicate-pct      (gauge-fn storage-metrics-registry ["duplicate-pct"]
+                                 (fn []
+                                   (let [dupes (value (:duplicate-catalog performance-metrics))
+                                         new   (value (:updated-catalog performance-metrics))]
+                                     (float (kitchensink/quotient dupes (+ dupes new))))))
+   :catalog-volatility (histogram storage-metrics-registry ["catalog-volitilty"])
 
-   :replace-facts     (timer [ns-str "default" "replace-facts-time"])
+   :replace-facts     (timer storage-metrics-registry ["replace-facts-time"])
 
-   :store-report      (timer [ns-str "default" "store-report-time"])
-   })
+   :store-report      (timer storage-metrics-registry ["store-report-time"])})
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Certname querying/deleting
@@ -335,33 +334,76 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Catalog updates/changes
 
-(defn catalog-metadata
-  "Returns the id and hash of certname's catalog"
+(defn latest-catalog-metadata
+  "Returns the id, hash, certname_id and producer_timestamp of certname's
+  catalog."
   [certname]
   {:pre [certname]}
   (jdbc/query-with-resultset
-   [(format "SELECT id, %s AS hash FROM catalogs WHERE certname=?"
+   [(format (str "SELECT catalogs.id AS catalog_id, certnames.id AS certname_id, "
+                 " %s AS catalog_hash, catalogs.producer_timestamp"
+                 " FROM certnames "
+                 " LEFT JOIN latest_catalogs ON certnames.id = latest_catalogs.certname_id"
+                 " LEFT JOIN catalogs ON catalogs.id = latest_catalogs.catalog_id"
+                 " WHERE certnames.certname=?")
             (sutils/sql-hash-as-str "catalogs.hash"))
     certname]
    (comp first sql/result-set-seq)))
 
+;; `historical-catalogs-limit` is used for toggling historical catalog
+;; storage, this is configurable and PE only.
+(def historical-catalogs-limit (atom 0))
+
+;; `store-catalogs-jsonb-columns?` is used for toggling storage of the resources
+;; and edges jsonb blobs for catalogs. These blobs are used in PE only and this
+;; variable is meant to only be set to true in PE. This exists so that we can
+;; store the jsonb columns idependently from storing historical catalogs. This
+;; way a user can turn off historical catalogs and the PE only views still work.
+(def store-catalogs-jsonb-columns? (atom false))
+
+(defn munge-edges-for-storage [edges]
+  (->> edges
+       (map (fn [{:keys [source target relationship]}]
+              {:source_type (:type source)
+               :source_title (:title source)
+               :target_type (:type target)
+               :target_title (:title target)
+               :relationship relationship}))
+       sutils/munge-jsonb-for-storage))
+
+(defn munge-resources-for-storage [resources]
+  (->> resources
+       (map (partial merge {:file nil :line nil}))
+       sutils/munge-jsonb-for-storage))
+
 (pls/defn-validated catalog-row-map
   "Creates a row map for the catalogs table, optionally adding envrionment when it was found"
   [hash
-   {:keys [version code_id transaction_uuid environment producer_timestamp]} :- catalog-schema
+   {:keys [edges
+           resources
+           version
+           code_id
+           transaction_uuid
+           catalog_uuid
+           environment
+           producer_timestamp]} :- catalog-schema
    received-timestamp :- pls/Timestamp]
-  {:hash (sutils/munge-hash-for-storage hash)
-   :catalog_version  version
-   :transaction_uuid (sutils/munge-uuid-for-storage transaction_uuid)
-   :timestamp (to-timestamp received-timestamp)
-   :code_id code_id
-   :environment_id (ensure-environment environment)
-   :producer_timestamp (to-timestamp producer_timestamp)
-   :api_version 1})
+  (let [catalogs-jsonb? @store-catalogs-jsonb-columns?]
+    {:hash (sutils/munge-hash-for-storage hash)
+     :edges (when catalogs-jsonb? (munge-edges-for-storage edges))
+     :resources (when catalogs-jsonb? (munge-resources-for-storage (vals resources)))
+     :catalog_version  version
+     :transaction_uuid (sutils/munge-uuid-for-storage transaction_uuid)
+     :catalog_uuid (sutils/munge-uuid-for-storage catalog_uuid)
+     :timestamp (to-timestamp received-timestamp)
+     :code_id code_id
+     :environment_id (ensure-environment environment)
+     :producer_timestamp (to-timestamp producer_timestamp)
+     :api_version 1}))
 
 (pls/defn-validated update-catalog-metadata!
   "Given some catalog metadata, update the db"
-  [id :- Number
+  [id :- Long
    hash :- String
    catalog :- catalog-schema
    received-timestamp :- pls/Timestamp]
@@ -400,13 +442,13 @@
 ;;should be changed to the correct schema
 (pls/defn-validated catalog-resources
   "Returns the resource hashes keyed by resource reference"
-  [catalog-id :- Number]
+  [certname-id :- Long]
   (jdbc/query-with-resultset
    [(format "SELECT type, title, tags, exported, file, line, %s
                AS resource
                FROM catalog_resources
-               WHERE catalog_id = ?"
-            (sutils/sql-hash-as-str "resource")) catalog-id]
+               WHERE certname_id = ?"
+            (sutils/sql-hash-as-str "resource")) certname-id]
    (fn [rs]
      (let [rss (sql/result-set-seq rs)]
        (zipmap (map #(select-keys % [:type :title]) rss)
@@ -467,7 +509,7 @@
 
 (pls/defn-validated insert-catalog-resources!
   "Returns a function that accepts a seq of ref keys to insert"
-  [catalog-id :- Number
+  [certname-id :- Long
    refs-to-hashes :- {resource-ref-schema String}
    refs-to-resources :- resource-ref->resource-schema]
   (fn [refs-to-insert]
@@ -480,7 +522,7 @@
      (map (fn [resource-ref]
             (let [{:keys [type title exported parameters tags file line] :as resource} (get refs-to-resources resource-ref)]
               (convert-tags-array
-               {:catalog_id catalog-id
+               {:certname_id certname-id
                 :resource (sutils/munge-hash-for-storage (get refs-to-hashes resource-ref))
                 :type type
                 :title title
@@ -492,7 +534,7 @@
 
 (pls/defn-validated delete-catalog-resources!
   "Returns a function accepts old catalog resources that should be deleted."
-  [catalog-id :- Number]
+  [certname-id :- Long]
   (fn [refs-to-delete]
     {:pre [(every? resource-ref? refs-to-delete)]}
 
@@ -500,8 +542,8 @@
 
     (doseq [{:keys [type title]} refs-to-delete]
       (jdbc/delete! :catalog_resources
-                    ["catalog_id = ? and type = ? and title = ?"
-                     catalog-id type title]))))
+                    ["certname_id = ? and type = ? and title = ?"
+                     certname-id type title]))))
 
 (s/defn basic-diff
   "Basic diffing that returns only the keys/values of `right` whose values don't match those of `left`.
@@ -535,7 +577,7 @@
 
 (pls/defn-validated update-catalog-resources!
   "Returns a function accepting keys that were the same from the old resources and the new resources."
-  [catalog-id :- Number
+  [certname-id :- Long
    refs-to-hashes :- {resource-ref-schema String}
    refs-to-resources
    old-resources]
@@ -551,8 +593,8 @@
       (doseq [[{:keys [type title]} updated-cols] updated-resources]
         (jdbc/update! :catalog_resources
                       (convert-tags-array updated-cols)
-                      ["catalog_id = ? and type = ? and title = ?"
-                       catalog-id type title])))))
+                      ["certname_id = ? and type = ? and title = ?"
+                       certname-id type title])))))
 
 (defn strip-params
   "Remove params from the resource as it is stored (and hashed) separately
@@ -562,18 +604,18 @@
 
 (pls/defn-validated add-resources!
   "Persist the given resource and associate it with the given catalog."
-  [catalog-id :- Number
+  [certname-id :- Long
    refs-to-resources :- resource-ref->resource-schema
    refs-to-hashes :- {resource-ref-schema String}]
-  (let [old-resources (catalog-resources catalog-id)
+  (let [old-resources (catalog-resources certname-id)
         diffable-resources (kitchensink/mapvals strip-params refs-to-resources)]
     (jdbc/with-db-transaction []
      (add-params! refs-to-resources refs-to-hashes)
      (utils/diff-fn old-resources
                     diffable-resources
-                    (delete-catalog-resources! catalog-id)
-                    (insert-catalog-resources! catalog-id refs-to-hashes diffable-resources)
-                    (update-catalog-resources! catalog-id refs-to-hashes diffable-resources old-resources)))))
+                    (delete-catalog-resources! certname-id)
+                    (insert-catalog-resources! certname-id refs-to-hashes diffable-resources)
+                    (update-catalog-resources! certname-id refs-to-hashes diffable-resources old-resources)))))
 
 (pls/defn-validated catalog-edges-map
   "Return all edges for a given catalog id as a map"
@@ -594,22 +636,26 @@
   "Delete edges for a given certname.
 
   Edges must be either nil or a collection of lists containing each element
-  of an edge, eg:
+  of an edge, e.g.:
 
     [[<source> <target> <type>] ...]"
   [certname :- String
    edges :- edge-db-schema]
 
   (update! (:catalog-volatility performance-metrics) (count edges))
-
+  
   (doseq [[source target type] edges]
     ;; This is relatively inefficient. If we have id's for edges, we could do
     ;; this in 1 statement.
     (jdbc/delete! :edges
-                  [(format "certname=? and %s=? and %s=? and type=?"
-                           (sutils/sql-hash-as-str "source")
-                           (sutils/sql-hash-as-str "target"))
-                   certname source target type])))
+                  [(str "certname=?"
+                        " and source=?::bytea"
+                        " and target=?::bytea"
+                        " and type=?")
+                   certname
+                   (sutils/bytea-escape source)
+                   (sutils/bytea-escape target)
+                   type])))
 
 (pls/defn-validated insert-edges!
   "Insert edges for a given certname.
@@ -659,10 +705,10 @@
                    #(delete-edges! certname %)
                    identity)))
 
-(pls/defn-validated update-catalog-hash-match
-  "When a new incoming catalog has the same hash as an existing catalog, update performance-metrics
-   and the transaction id for the new catalog"
-  [catalog-id :- Number
+(pls/defn-validated update-existing-catalog
+  "When a new incoming catalog has the same hash as an existing catalog, update
+   performance-metrics and the transaction id for the new catalog"
+  [catalog-id :- Long
    hash :- String
    catalog :- catalog-schema
    received-timestamp :- pls/Timestamp]
@@ -672,81 +718,121 @@
 
 (pls/defn-validated update-catalog-associations!
   "Adds/updates/deletes the edges and resources for the given certname"
-  [catalog-id :- Number
+  [certname-id :- Long
    {:keys [resources edges certname]} :- catalog-schema
    refs-to-hashes :- {resource-ref-schema String}]
   (time! (:add-resources performance-metrics)
-         (add-resources! catalog-id resources refs-to-hashes))
+         (add-resources! certname-id resources refs-to-hashes))
   (time! (:add-edges performance-metrics)
          (replace-edges! certname edges refs-to-hashes)))
 
-(pls/defn-validated update-catalog-hash-miss
-  "New catalogs for a given certname needs to have their metadata, resources and edges updated.  This
-   function also outputs debugging related information when `catalog-hash-debug-dir` is not nil"
-  [catalog-id :- Number
+(pls/defn-validated replace-existing-catalog
+  "New catalogs for a given certname needs to have their metadata, resources and
+  edges updated."
+  [certname-id :- Long
+   catalog-id :- Long
    hash :- String
    catalog :- catalog-schema
    refs-to-hashes :- {resource-ref-schema String}
-   catalog-hash-debug-dir :- (s/maybe s/Str)
    received-timestamp :- pls/Timestamp]
 
   (inc! (:updated-catalog performance-metrics))
 
-  (when catalog-hash-debug-dir
-    (hashdbg/debug-catalog catalog-hash-debug-dir hash catalog))
-
   (time! (:catalog-hash-miss performance-metrics)
          (update-catalog-metadata! catalog-id hash catalog received-timestamp)
-         (update-catalog-associations! catalog-id catalog refs-to-hashes)))
+         (update-catalog-associations! certname-id catalog refs-to-hashes)))
 
 (pls/defn-validated add-new-catalog
   "Creates new catalog metadata and adds the proper associations for the edges and resources"
-  [hash :- String
+  [certname-id :- Long
+   hash :- String
    catalog :- catalog-schema
    refs-to-hashes :- {resource-ref-schema String}
    received-timestamp :- pls/Timestamp]
   (inc! (:updated-catalog performance-metrics))
   (time! (:add-new-catalog performance-metrics)
          (let [catalog-id (:id (add-catalog-metadata! hash catalog received-timestamp))]
-           (update-catalog-associations! catalog-id catalog refs-to-hashes))))
+           (jdbc/insert! :latest_catalogs {:certname_id certname-id :catalog_id catalog-id})
+           (update-catalog-associations! certname-id catalog refs-to-hashes))))
 
 (pls/defn-validated add-catalog!
   "Persist the supplied catalog in the database, returning its
-   similarity hash. `catalog-hash-debug-dir` is an optional path that
-   indicates where catalog debugging information should be stored."
+   similarity hash."
+  [{:keys [producer_timestamp resources certname] :as catalog} :- catalog-schema
+   received-timestamp :- pls/Timestamp
+   historical-catalogs-limit]
+  (time! (:replace-catalog performance-metrics)
+         (jdbc/with-db-transaction []
+           (let [hash (time! (:catalog-hash performance-metrics)
+                             (shash/catalog-similarity-hash catalog))
+                 {certname-id :certname_id
+                  latest-producer-timestamp :producer_timestamp} (latest-catalog-metadata certname)]
+             (inc! (:updated-catalog performance-metrics))
+             (time! (:add-new-catalog performance-metrics)
+                    (let [catalog-id (:id (add-catalog-metadata! hash catalog received-timestamp))]
+
+                      (when-not (some-> latest-producer-timestamp
+                                        (.after (to-timestamp producer_timestamp)))
+                        (let [refs-to-hashes (time! (:resource-hashes performance-metrics)
+                                                    (kitchensink/mapvals shash/resource-identity-hash resources))]
+                          (if-not latest-producer-timestamp
+                            (jdbc/insert! :latest_catalogs {:certname_id certname-id :catalog_id catalog-id})
+                            (jdbc/update! :latest_catalogs {:catalog_id catalog-id} ["certname_id=?" certname-id]))
+                          (update-catalog-associations! certname-id catalog refs-to-hashes)))
+                      (jdbc/delete! :catalogs
+                                    [(str "certname = ? AND "
+                                          "id NOT IN (SELECT id FROM catalogs "
+                                          "           WHERE certname=?"
+                                          "           ORDER BY producer_timestamp DESC LIMIT ?)")
+                                     certname
+                                     certname
+                                     historical-catalogs-limit])))
+             hash))))
+
+(pls/defn-validated replace-catalog!
+  "Persist the supplied catalog in the database, returning its
+   similarity hash."
   ([catalog :- catalog-schema]
-   (add-catalog! catalog nil (now)))
-  ([{:keys [api_version resources edges certname] :as catalog} :- catalog-schema
-    catalog-hash-debug-dir :- (s/maybe s/Str)
+   (replace-catalog! catalog (now)))
+  ([{:keys [producer_timestamp resources certname] :as catalog} :- catalog-schema
     received-timestamp :- pls/Timestamp]
+   (time! (:replace-catalog performance-metrics)
+          (jdbc/with-db-transaction []
+            (let [hash (time! (:catalog-hash performance-metrics)
+                              (shash/catalog-similarity-hash catalog))
+                  {catalog-id :catalog_id
+                   stored-hash :catalog_hash
+                   certname-id :certname_id
+                   latest-producer-timestamp :producer_timestamp} (latest-catalog-metadata certname)]
+              (cond
+                (some-> latest-producer-timestamp
+                        (.after (to-timestamp producer_timestamp)))
+                (log/warnf "Not replacing catalog for certname %s because local data is newer." certname)
 
-   (let [refs-to-hashes (time! (:resource-hashes performance-metrics)
-                               (reduce-kv (fn [acc k v]
-                                            (assoc acc k (shash/resource-identity-hash v)))
-                                          {} resources))
-         hash (time! (:catalog-hash performance-metrics)
-                     (shash/catalog-similarity-hash catalog))
-         {id :id
-          stored-hash :hash} (catalog-metadata certname)]
+                (= stored-hash hash)
+                (update-existing-catalog catalog-id hash catalog received-timestamp)
 
-     (jdbc/with-db-transaction []
-      (cond
-       (nil? id)
-       (add-new-catalog hash catalog refs-to-hashes received-timestamp)
+                :else
+                (let [refs-to-hashes (time! (:resource-hashes performance-metrics)
+                                            (kitchensink/mapvals shash/resource-identity-hash resources))]
+                  (if (nil? catalog-id)
+                    (add-new-catalog certname-id hash catalog refs-to-hashes received-timestamp)
+                    (replace-existing-catalog certname-id catalog-id hash catalog refs-to-hashes received-timestamp))))
+              hash)))))
 
-       (= stored-hash hash)
-       (update-catalog-hash-match id hash catalog received-timestamp)
+(pls/defn-validated store-catalog!
+  "Persist the supplied catalog in the database, returning its similarity hash.
+   On PE we alter the atom 'historical-catalog-limit' to be >0 and we will store
+   the catalog in database alongside the other catalogs for the node. On FOSS we
+   will not store historical catalogs and will replace the existing catalog for
+   node."
+  [catalog :- catalog-schema
+   received-timestamp :- pls/Timestamp]
+  (let [historical-catalogs-limit @historical-catalogs-limit]
+    (if (> historical-catalogs-limit 0)
+      (add-catalog! catalog received-timestamp historical-catalogs-limit)
+      (replace-catalog! catalog received-timestamp))))
 
-       :else
-       (update-catalog-hash-miss id hash catalog refs-to-hashes catalog-hash-debug-dir received-timestamp)))
-
-     hash)))
-
-(defn delete-catalog!
-  "Remove the catalog identified by the following hash"
-  [catalog-hash]
-  (jdbc/delete! :catalogs [(str (sutils/sql-hash-as-str "hash") "=?")
-                           catalog-hash]))
 
 (defn catalog-hash-for-certname
   "Returns the hash for the `certname` catalog"
@@ -782,13 +868,6 @@
            UNION SELECT environment_id FROM factsets
                    WHERE environment_id IS NOT NULL)"])))
 
-(defn delete-unassociated-statuses!
-  "Remove any statuses that aren't associated with a report"
-  []
-  (time! (:gc-report-statuses performance-metrics)
-         (jdbc/delete! :report_statuses
-                       ["ID NOT IN (SELECT status_id FROM reports)"])))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Facts
 
@@ -813,38 +892,22 @@
    in other factsets."
   [factset-id dropped-pids]
   (when-let [dropped-pids (seq dropped-pids)]
-    (let [dropped-chunks (partition-all gc-chunksize dropped-pids)]
-      (if (sutils/postgres?)
-        (let [candidate-chunks (map (partial sutils/array-to-param "bigint" Long)
-                                    dropped-chunks)]
-          (dorun
-            (map (fn [candidate-paths]
-                   (jdbc/do-prepared
-                     "DELETE FROM fact_paths fp
+    (let [dropped-chunks (partition-all gc-chunksize dropped-pids)
+          candidate-chunks (map (partial sutils/array-to-param "bigint" Long)
+                                dropped-chunks)]
+      (dorun
+       (map (fn [candidate-paths]
+              (jdbc/do-prepared
+               "DELETE FROM fact_paths fp
                         WHERE fp.id = ANY (?)
                         AND NOT EXISTS (SELECT 1 FROM facts f
                                         WHERE f.fact_path_id = ANY(?)
                                         AND f.fact_path_id = fp.id
                                         AND f.factset_id <> ?)"
-                     (concat [candidate-paths]
-                             [candidate-paths]
-                             [factset-id])))
-                 candidate-chunks)))
-        (let [in-chunks (map jdbc/in-clause dropped-chunks)]
-          (dorun
-            (map (fn [in dropped]
-                   (jdbc/do-prepared
-                     (format
-                       "DELETE FROM fact_paths fp
-                          WHERE fp.id %s
-                          AND NOT EXISTS (SELECT 1 FROM facts f
-                                          WHERE f.fact_path_id %s
-                                          AND f.fact_path_id = fp.id
-                                          AND f.factset_id <> ?)"
-                       in in)
-                     (concat dropped dropped [factset-id])))
-                 in-chunks
-                 dropped-chunks)))))))
+               (concat [candidate-paths]
+                       [candidate-paths]
+                       [factset-id])))
+            candidate-chunks)))))
 
 (defn-validated delete-pending-value-id-orphans!
   "Delete values in removed-pid-vid-pairs that are no longer mentioned
@@ -886,7 +949,7 @@
   transaction may decide not to delete paths that are only referred to
   by other facts that are being changed in parallel transactions to
   also not refer to the paths."
-  [n :- (s/both s/Int (s/pred (complement neg?) 'nonnegative?))]
+  [n :- (s/constrained s/Int (complement neg?))]
   (if (zero? n)
     0
     (first
@@ -908,7 +971,7 @@
   read, an update transaction may decide not to delete values that are
   only referred to by other facts that are being changed in parallel
   transactions to also not refer to the values."
-  [n :- (s/both s/Int (s/pred (complement neg?) 'nonnegative?))]
+  [n :- (s/constrained s/Int (complement neg?))]
   (if (zero? n)
     0
     (first
@@ -938,8 +1001,8 @@
 
 (defn-validated insert-facts-pv-pairs!
   [factset-id :- s/Int
-   pairs :- (s/either [(s/pair s/Int "path-id" s/Int "value-id")]
-                      #{(s/pair s/Int "path-id" s/Int "value-id")})]
+   pairs :- (s/cond-pre [(s/pair s/Int "path-id" s/Int "value-id")]
+                        #{(s/pair s/Int "path-id" s/Int "value-id")})]
   (when-let [rows (seq (for [[pid vid] pairs]
                          {:factset_id factset-id
                           :fact_path_id pid
@@ -952,17 +1015,13 @@
   [table column values]
   (into {}
         (for [{:keys [value id]}
-              (if (sutils/postgres?)
-                (query-to-vec (format
-                               "WITH values AS (SELECT unnest(?) as vals)
-                                SELECT %s AS value, id FROM %s
-                                INNER JOIN values ON values.vals = %s"
-                               column table column)
-                              (sutils/array-to-param "text" String values))
-                (apply query-to-vec
-                       (format "SELECT %s AS value, id FROM %s WHERE %s %s"
-                               column table column (jdbc/in-clause values))
-                       values))]
+              (query-to-vec
+               (format
+                "WITH values AS (SELECT unnest(?) as vals)
+                   SELECT %s AS value, id FROM %s
+                   INNER JOIN values ON values.vals = %s"
+                column table column)
+               (sutils/array-to-param "text" String values))]
           [value id])))
 
 (defn realize-records!
@@ -1181,13 +1240,17 @@
   (time! (:store-report performance-metrics)
          (let [{:keys [puppet_version certname report_format configuration_version
                        producer_timestamp start_time end_time transaction_uuid environment
-                       status noop metrics logs resources resource_events]
+                       status noop metrics logs resources resource_events catalog_uuid
+                       code_id cached_catalog_status]
                 :as report} (normalize-report orig-report)
                 report-hash (shash/report-identity-hash report)]
            (jdbc/with-db-transaction []
              (let [certname-id (certname-id certname)
                    row-map {:hash (sutils/munge-hash-for-storage report-hash)
                             :transaction_uuid (sutils/munge-uuid-for-storage transaction_uuid)
+                            :catalog_uuid (sutils/munge-uuid-for-storage catalog_uuid)
+                            :code_id code_id
+                            :cached_catalog_status cached_catalog_status
                             :metrics (sutils/munge-jsonb-for-storage metrics)
                             :logs (sutils/munge-jsonb-for-storage logs)
                             :resources (sutils/munge-jsonb-for-storage resources)
@@ -1222,33 +1285,18 @@
    the specified date/time."
   [time]
   {:pre [(kitchensink/datetime? time)]}
-  (when (not (sutils/postgres?))
-    ;; there's an ON DELETE SET NULL foreign key constraint in postgres for
-    ;; this, but we can't do that in hsqldb
-    (jdbc/update! :certnames
-                  {:latest_report_id nil}
-                  ["latest_report_id in (select id from reports where producer_timestamp < ?)"
-                   (to-timestamp time)]))
   (jdbc/delete! :reports ["producer_timestamp < ?" (to-timestamp time)]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Database support/deprecation
 
-(defn db-deprecated-msg
-  "Returns a string with a deprecation message if the DB is deprecated,
-   nil otherwise."
-  []
-  (when-not (sutils/postgres?)
-    (str "HSQLDB support has been deprecated and will be removed in a future version. "
-         "Please migrate to PostgreSQL.")))
-
 (defn db-unsupported-msg
   "Returns a string with an unsupported message if the DB is not supported,
   nil otherwise."
   []
-  (when (and (sutils/postgres?)
-             (sutils/db-version-older-than? [9 4]))
-    "PostgreSQL DB versions older than 9.4 are no longer supported. Please upgrade Postgres and restart PuppetDB."))
+  (when (sutils/db-version-older-than? [9 4])
+    (str "PostgreSQL DB versions older than 9.4 are no longer supported."
+         "  Please upgrade Postgres and restart PuppetDB.")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
@@ -1325,28 +1373,6 @@
                          WHERE certname=? AND expired IS NULL"
                       [timestamp certname])))
 
-(pls/defn-validated catalog-newer-than?
-  "Returns true if the most current catalog for `certname` is more recent than
-  `time`."
-  [certname :- String
-   producer-timestamp :- pls/Timestamp]
-  (have-record-produced-after? :catalogs certname producer-timestamp))
-
-(pls/defn-validated replace-catalog!
-  "Given a catalog, replace the current catalog, if any, for its
-  associated host with the supplied one. `catalog-hash-debug-dir`
-  is an optional path that indicates where catalog debugging information
-  should be stored."
-  ([catalog :- catalog-schema
-    received-timestamp :- pls/Timestamp]
-   (replace-catalog! catalog received-timestamp nil))
-  ([{:keys [certname] :as catalog} :- catalog-schema
-    received-timestamp :- pls/Timestamp
-    catalog-hash-debug-dir :- (s/maybe s/Str)]
-   (time! (:replace-catalog performance-metrics)
-          (jdbc/with-db-transaction []
-            (add-catalog! catalog catalog-hash-debug-dir received-timestamp)))))
-
 (pls/defn-validated replace-facts!
   "Updates the facts of an existing node, if the facts are newer than the current set of facts.
    Adds all new facts if no existing facts are found. Invoking this function under the umbrella of
@@ -1376,9 +1402,7 @@
     (let [msg (utils/attention-msg msg)]
       (utils/println-err msg)
       (log/error msg)
-      (fail-fn)))
-  (when-let [msg (db-deprecated-msg)]
-    (log/warn msg)))
+      (fail-fn))))
 
 (def ^:dynamic *orphaned-path-gc-limit* 200)
 (def ^:dynamic *orphaned-value-gc-limit* 200)
@@ -1390,8 +1414,7 @@
    (:gc performance-metrics)
    (jdbc/with-transacted-connection db
      (delete-unassociated-params!)
-     (delete-unassociated-environments!)
-     (delete-unassociated-statuses!))
+     (delete-unassociated-environments!))
    ;; These require serializable because they make the decision to
    ;; delete based on row counts in another table.
    (jdbc/with-transacted-connection' db :serializable
