@@ -1,10 +1,14 @@
 (ns puppetlabs.puppetdb.scf.storage-utils
   (:require [clojure.java.jdbc :as sql]
+            [clojure.tools.logging :as log]
             [honeysql.core :as hcore]
+            [honeysql.format :as hfmt]
+            [clojure.string :as str]
             [puppetlabs.puppetdb.cheshire :as json]
             [puppetlabs.puppetdb.honeysql :as h]
             [puppetlabs.puppetdb.jdbc :as jdbc]
             [puppetlabs.puppetdb.schema :as pls]
+            [puppetlabs.puppetdb.utils :as utils]
             [puppetlabs.kitchensink.core :as kitchensink]
             [schema.core :as s])
   (:import [org.postgresql.util PGobject]))
@@ -46,11 +50,6 @@
 (def db-metadata
   (delay (db-metadata-fn)))
 
-(pls/defn-validated postgres? :- s/Bool
-  "Returns true if currently connected to a Postgres DB instance"
-  []
-  (= (:database @db-metadata) "PostgreSQL"))
-
 (pls/defn-validated db-version? :- s/Bool
   "Returns true if the version list you pass matches the version of the current
    database."
@@ -89,11 +88,27 @@
         results (jdbc/with-db-transaction [] (jdbc/query-to-vec query))]
     (map :sequence_name results)))
 
+(defn sql-current-connection-function-names
+  "Returns the names of all of the functions in the public schema of
+  the current connection's database.  This is most useful for
+  debugging / testing purposes to allow introspection on the
+  database.  (Some of our unit tests rely on this.)."
+  []
+  (let [query (str "SELECT pp.proname as name, pg_catalog.pg_get_function_arguments(pp.oid) as args "
+                   "FROM pg_proc pp "
+                   "INNER JOIN pg_namespace pn ON (pp.pronamespace = pn.oid) "
+                   "INNER JOIN pg_language pl ON (pp.prolang = pl.oid) "
+                   "WHERE pl.lanname NOT IN ('c') "
+                   "AND pn.nspname NOT LIKE 'pg_%'"
+                   "AND pn.nspname <> 'information_schema'")
+        results (jdbc/with-db-transaction [] (jdbc/query-to-vec query))]
+    (map (fn [{:keys [name args]}] (str name "(" args ")"))
+         results)))
+
 (pls/defn-validated pg-installed-extensions :- {s/Str pg-extension-map}
   "Obtain the extensions installed and metadata about each extension for
    the current database."
   []
-  {:pre [(postgres?)]}
   (let [query "SELECT extname as name,
                       extversion as version,
                       extrelocatable as relocatable
@@ -105,17 +120,23 @@
 (pls/defn-validated pg-extension? :- s/Bool
   "Returns true if the named PostgreSQL extension is installed."
   [extension :- s/Str]
-  {:pre [(postgres?)]}
   (let [extensions (pg-installed-extensions)]
     (not= (get extensions extension) nil)))
 
+(defn current-schema
+  "Returns the current schema of the database connection on postgres."
+  []
+  (->> (jdbc/query-to-vec "select current_schema")
+       first
+       :current_schema))
+
 (pls/defn-validated index-exists? :- s/Bool
   "Returns true if the index exists. Only supported on PostgreSQL currently."
   ([index :- s/Str]
-     (index-exists? index "public"))
+   (let [schema (current-schema)]
+     (index-exists? index schema)))
   ([index :- s/Str
     namespace :- s/Str]
-     {:pre [(postgres?)]}
      (let [query "SELECT c.relname
                     FROM   pg_index as idx
                     JOIN   pg_class as c ON c.oid = idx.indexrelid
@@ -134,110 +155,57 @@
        (into-array Object)
        (.createArrayOf (:connection (jdbc/db)) "varchar")))
 
-(defmulti sql-array-type-string
-  "Returns a string representing the correct way to declare an array
-  of the supplied base database type."
-  ;; Dispatch based on database from the metadata of DB connection at the time
-  ;; of call; this copes gracefully with multiple connection types.
-  (fn [_] (:database @db-metadata)))
-
-(defmulti sql-array-query-string
-  "Returns an SQL fragment representing a query for a single value being
-found in an array column in the database.
-
-  `(str \"SELECT ... WHERE \" (sql-array-query-string \"column_name\"))`
-
-The returned SQL fragment will contain *one* parameter placeholder, which
-must be supplied as the value to be matched."
-  (fn [column] (:database @db-metadata)))
-
-(defmulti legacy-sql-regexp-match
-  "Returns db-specific code for performing a regexp match"
-  (fn [_] (:database @db-metadata)))
-
-(defmethod legacy-sql-regexp-match "PostgreSQL"
+(defn legacy-sql-regexp-match
+  "Returns the SQL for performing a regexp match."
   [column]
   (format "(%s ~ ? AND %s IS NOT NULL)" column column))
 
-(defmethod legacy-sql-regexp-match "HSQL Database Engine"
-  [column]
-  (format "REGEXP_SUBSTRING(%s, ?) IS NOT NULL" column))
-
-(defmulti sql-regexp-match
-  "Returns db-specific code for performing a regexp match"
-  (fn [_] (:database @db-metadata)))
-
-(defmulti sql-regexp-array-match
-  "Returns db-specific code for performing a regexp match against the
-  contents of an array. If any of the array's items match the supplied
-  regexp, then that satisfies the match."
-  (fn [_ _ _] (:database @db-metadata)))
-
-(defmulti sql-as-numeric
-  "Returns appropriate db-specific code for converting the given column to a
-  number, or to NULL if it is not numeric."
-  (fn [_] (:database @db-metadata)))
-
-(defmethod sql-as-numeric "PostgreSQL"
+(defn sql-as-numeric
+  "Returns the SQL for converting the given column to a number, or to
+  NULL if it is not numeric."
   [column]
   (hcore/raw (format (str "CASE WHEN %s~E'^\\\\d+$' THEN %s::bigint "
                           "WHEN %s~E'^\\\\d+\\\\.\\\\d+$' THEN %s::float "
                           "ELSE NULL END")
                      column column column column)))
 
-(defmethod sql-as-numeric "HSQL Database Engine"
-  [column]
-  (hcore/raw (format (str "CASE WHEN REGEXP_MATCHES(%s, '^\\d+$') THEN CAST(%s AS BIGINT) "
-                          "WHEN REGEXP_MATCHES(%s, '^\\d+\\.\\d+$') THEN CAST(%s AS FLOAT) "
-                          "ELSE NULL END")
-                     column column column column)))
-
-(defmethod sql-array-type-string "PostgreSQL"
+(defn sql-array-type-string
+  "Returns the SQL to declare an array of the supplied base database
+  type."
   [basetype]
   (format "%s ARRAY[1]" basetype))
 
-(defmethod sql-array-type-string "HSQL Database Engine"
-  [basetype]
-  (format "%s ARRAY[%d]" basetype 65535))
+(defn sql-array-query-string
+  "Returns an SQL fragment representing a query for a single value
+  being found in an array column in the database.
 
-(defmethod sql-array-query-string "PostgreSQL"
+    (str \"SELECT ... WHERE \" (sql-array-query-string \"column_name\"))
+
+  The returned SQL fragment will contain *one* parameter placeholder,
+  which must be supplied as the value to be matched."
   [column]
   (hcore/raw
    (format "ARRAY[?::text] <@ %s" (name column))))
 
-(defmethod sql-array-query-string "HSQL Database Engine"
-  [column]
-  (hcore/raw
-   (format "? IN (UNNEST(%s))" (name column))))
-
-(defmethod sql-regexp-match "PostgreSQL"
+(defn sql-regexp-match
+  "Returns db code for performing a regexp match."
   [column]
   [:and
    [(keyword "~") column "?"]
    [:is-not column nil]])
 
-(defmethod sql-regexp-match "HSQL Database Engine"
+(defn sql-regexp-array-match
+  "Returns SQL for performing a regexp match against the contents of
+  an array. If any of the array's items match the supplied regexp,
+  then that satisfies the match."
   [column]
-  [:is-not (h/regexp-substring column "?") nil])
-
-(defmethod sql-regexp-array-match "PostgreSQL"
-  [orig-table _ column]
   (hcore/raw
    (format "EXISTS(SELECT 1 FROM UNNEST(%s) WHERE UNNEST ~ ?)" (name column))))
 
-(defmethod sql-regexp-array-match "HSQL Database Engine"
-  [orig-table query-table column]
-  ;; What evil have I wrought upon the land? Good gravy.
-  ;;
-  ;; This is entirely due to the fact that HSQLDB doesn't support the
-  ;; UNNEST operator referencing a column from an outer table. UNNEST
-  ;; *has* to come after the parent table in the FROM clause of a
-  ;; separate SQL statement.
-  (let [col (name column)]
-    (hcore/raw
-     (format (str "EXISTS(SELECT 1 FROM %s %s_copy, UNNEST(%s) AS T(the_tag) "
-                  "WHERE %s.%s=%s_copy.%s AND REGEXP_SUBSTRING(the_tag, ?) IS NOT NULL)")
-             orig-table orig-table col query-table col orig-table col))))
+(defn sql-in-array
+  [column]
+  (hcore/raw
+   (format "%s = ANY(?)" (first (hfmt/format column)))))
 
 (defn db-serialize
   "Serialize `value` into a form appropriate for querying against a
@@ -252,64 +220,35 @@ must be supplied as the value to be matched."
   {:pre [(string? table)
          (string? column)]}
   (jdbc/with-db-transaction []
-   (if (postgres?)
-     ;; PostgreSQL specific way
-     (do
-       (jdbc/do-commands (str "LOCK TABLE " table " IN ACCESS EXCLUSIVE MODE"))
-       (jdbc/query-with-resultset
-        [(str "SELECT setval(pg_get_serial_sequence(?, ?),
-                             (SELECT max(" column ") FROM " table "))")
-         table column]
-        (constantly nil)))
-
-     ;; HSQLDB specific way
-     (let [_ (jdbc/do-commands (str "LOCK TABLE " table " WRITE"))
-           maxid (jdbc/query-with-resultset
-                  [(str "SELECT max(" column ") as id FROM " table)]
-                  (comp :id first sql/result-set-seq))
-           ;; While postgres handles a nil case gracefully, hsqldb does not
-           ;; so here we return 1 if the maxid is nil, and otherwise return
-           ;; maxid +1 to indicate that the next number should be higher
-           ;; then the current one.
-           restartid (if (nil? maxid) 1 (inc maxid))]
-       (jdbc/do-commands (str "ALTER TABLE " table " ALTER COLUMN " column
-                              " RESTART WITH " restartid))))))
+    (jdbc/do-commands (str "LOCK TABLE " table " IN ACCESS EXCLUSIVE MODE"))
+    (jdbc/query-with-resultset
+     [(format
+       "select setval(pg_get_serial_sequence(?, ?), (select max(%s) from %s))"
+       column table)
+      table column]
+     (constantly nil))))
 
 (defn sql-hash-as-str
   [column]
-  (if (postgres?)
-    (format "encode(%s::bytea, 'hex')" column)
-    column))
-
-(defn sql-uuid-as-str
-  [column]
-  (if (postgres?)
-    (format "%s::text" column)
-    column))
+  (format "encode(%s::bytea, 'hex')" column))
 
 (defn vacuum-analyze
   [db]
-  (jdbc/do-commands false "vacuum analyze"))
+  (sql/with-db-connection [conn db]
+    (sql/execute! db ["vacuum analyze"] :transaction? false)))
 
 (defn parse-db-hash
   [db-hash]
-  (if (postgres?)
-    (clojure.string/replace (.getValue db-hash) "\\x" "")
-    db-hash))
+  (clojure.string/replace (.getValue db-hash) "\\x" ""))
 
 (defn parse-db-uuid
   [db-uuid]
-  (if (postgres?)
-    (.toString db-uuid)
-    db-uuid))
+  (.toString db-uuid))
 
 (pls/defn-validated parse-db-json
   "Produce a function for parsing an object stored as json."
-  [db-json :- (s/maybe (s/either s/Str PGobject))]
-  (if-let [json (if (postgres?)
-                  (when db-json (.getValue db-json))
-                  db-json)]
-    (json/parse-string json true)))
+  [db-json :- (s/maybe (s/cond-pre s/Str PGobject))]
+  (some-> db-json .getValue (json/parse-string true)))
 
 (pls/defn-validated str->pgobject :- PGobject
   [type :- s/Str
@@ -320,29 +259,49 @@ must be supplied as the value to be matched."
 
 (defn munge-uuid-for-storage
   [value]
-  (if (postgres?)
-    (str->pgobject "uuid" value)
-    value))
+  (str->pgobject "uuid" value))
+
+(defn bytea-escape [s]
+  (format "\\x%s" s))
 
 (defn munge-hash-for-storage
   [hash]
-  (if (postgres?)
-    (str->pgobject "bytea" (format "\\x%s" hash))
-    hash))
+  (str->pgobject "bytea" (bytea-escape hash)))
 
 (defn munge-json-for-storage
   "Prepare a clojure object for storage depending on db type."
   [value]
   (let [json-str (json/generate-string value)]
-    (if (postgres?)
-      (str->pgobject "json" json-str)
-      json-str)))
+    (str->pgobject "json" json-str)))
 
 (defn munge-jsonb-for-storage
   "Prepare a clojure object for storage depending on db type."
   [value]
   (let [json-str (json/generate-string value)]
-    (if (postgres?)
-      (str->pgobject "jsonb" json-str)
-      json-str)))
+    (str->pgobject "jsonb" json-str)))
 
+(defn db-up?
+  [db-spec]
+  (utils/with-timeout 1000 false
+    (try
+      (jdbc/with-transacted-connection db-spec
+        (let [select-42 "SELECT (a - b) AS answer FROM (VALUES ((7 * 7), 7)) AS x(a, b)"
+              [{:keys [answer]}] (jdbc/query [select-42])]
+          (= answer 42)))
+      (catch Exception _
+        false))))
+
+(defn async-vacuum-analyze
+  "We don't want to stay in maintenance mode while we vaccum analyze,
+  i.e. refuse http requests as this can take quite some time on large
+  databases"
+  [db-connection-pool]
+  (future
+    (jdbc/with-db-connection db-connection-pool
+      (log/info "Analyzing database")
+      (try
+        (jdbc/do-commands-outside-txn "vacuum (analyze, verbose)")
+        (catch java.sql.SQLException e
+          (log/error e "Caught SQLException during migration")
+          (when-let [next (.getNextException e)]
+            (log/error next "Unravelled exception")))))))
