@@ -8,7 +8,6 @@
             [clojure.set :as set]
             [puppetlabs.puppetdb.archive :as archive]
             [clojure.java.io :as io]
-            [puppetlabs.puppetdb.scf.storage-utils :as sutils]
             [puppetlabs.puppetdb.cheshire :as json]
             [clojure.walk :as walk]
             [slingshot.slingshot :refer [try+ throw+]]
@@ -164,11 +163,7 @@
    :host s/Str
    :port s/Int
    (s/optional-key :prefix) s/Str
-   (s/optional-key :version) (s/both
-                              s/Keyword
-                              (s/pred #(and (keyword? %)
-                                            (re-matches #"v\d+" (name %)))
-                                      'valid-version?))})
+   (s/optional-key :version) (s/constrained s/Keyword #(re-matches #"v\d+" (name %)))})
 
 (pls/defn-validated base-url->str :- s/Str
   "Converts the `base-url' map to an ASCII URL.  May throw
@@ -185,6 +180,12 @@
   (-> (URL. protocol host port "")
       .toURI .toASCIIString))
 
+(defn base-url->str-with-prefix
+  [{:keys [protocol host port prefix] :as base-url}]
+  (-> (java.net.URL. protocol host port prefix)
+      .toURI
+      .toASCIIString))
+
 (defn describe-bad-base-url
   "If a problem is detected with `base-url`, returns a string
   describing the issue. For example {:host \"x:y\" ...}."
@@ -271,37 +272,6 @@
                (assoc acc (schema.core/required-key (puppetlabs.puppetdb.utils/kwd->str k)) v))
              {} kwd-schema))
 
-(defn as-path
-  "Create a url path from arguments. Does not append a slash to the beginning
-   or end. Example:
-   (as-path '/v4' 'facts' "
-  [root & path]
-  (apply str root "/" (string/join "/" path)))
-
-(pls/defn-validated child->expansion
-  "Convert child to the expanded format."
-  [data :- (s/maybe (s/either PGobject s/Str))
-   parent :- s/Keyword
-   child :- s/Keyword
-   url-prefix :- s/Str]
-  (let [to-href #(as-path url-prefix (name parent) % (name child))]
-    (if (string? data)
-      ;; if it's a string it's just an identifier
-      {:href (to-href data)}
-      (-> (sutils/parse-db-json data)
-          (update :href to-href)))))
-
-(defn hsql?
-  "given a db-spec style database object, determine if hsqldb is being used."
-  [db-spec]
-  (cond
-    (:subprotocol db-spec)
-    (= (:subprotocol db-spec) "hsqldb")
-
-    (:datasource db-spec)
-    (re-matches #"jdbc:hsqldb.*"
-                (.getJdbcUrl (:datasource db-spec)))))
-
 (defn dashes->underscores
   "Accepts a string or a keyword as an argument, replaces all occurrences of the
   dash/hyphen character with an underscore, and returns the same type (string
@@ -339,3 +309,16 @@
   (sp/transform [sp/ALL]
                 #(update % 0 underscores->dashes)
                 m))
+
+(defn cmd-params->json-str
+  [{:strs [command version certname payload]}]
+  (json/generate-string
+    {:command command :version (Integer. version)
+     :certname certname :payload (json/->RawJsonString payload)}))
+
+(defmacro with-timeout [timeout-ms default & body]
+  `(let [f# (future (do ~@body))
+         result# (deref f# ~timeout-ms ~default)]
+     result#))
+
+(def byte-array-class (Class/forName "[B"))
